@@ -1,4 +1,4 @@
-/* $Id: glu.h,v 1.7 1997/08/19 02:35:07 brianp Exp $ */
+/* $Id: glu.h,v 1.8 1997/10/29 02:03:20 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -23,6 +23,9 @@
 
 /*
  * $Log: glu.h,v $
+ * Revision 1.8  1997/10/29 02:03:20  brianp
+ * added WINGDIAPI, APIENTRY stuff (David Bucciarelli, v20 3dfx driver)
+ *
  * Revision 1.7  1997/08/19 02:35:07  brianp
  * added some Macintosh-only pragmas (Miklos Fazekas)
  *
@@ -212,36 +215,36 @@ typedef struct GLUnurbsObj GLUnurbsObj;
  *
  */
 
-extern void gluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
-		       GLdouble centerx, GLdouble centery, GLdouble centerz,
-		       GLdouble upx, GLdouble upy, GLdouble upz );
+extern void APIENTRY gluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
+                                GLdouble centerx, GLdouble centery, GLdouble centerz,
+                                GLdouble upx, GLdouble upy, GLdouble upz );
 
 
-extern void gluOrtho2D( GLdouble left, GLdouble right,
-		        GLdouble bottom, GLdouble top );
+extern void APIENTRY gluOrtho2D( GLdouble left, GLdouble right,
+                                 GLdouble bottom, GLdouble top );
 
 
-extern void gluPerspective( GLdouble fovy, GLdouble aspect,
-			    GLdouble zNear, GLdouble zFar );
+extern void APIENTRY gluPerspective( GLdouble fovy, GLdouble aspect,
+                                     GLdouble zNear, GLdouble zFar );
 
 
-extern void gluPickMatrix( GLdouble x, GLdouble y,
-			   GLdouble width, GLdouble height,
-			   const GLint viewport[4] );
+extern void APIENTRY gluPickMatrix( GLdouble x, GLdouble y,
+                                    GLdouble width, GLdouble height,
+                                    const GLint viewport[4] );
 
-extern GLint gluProject( GLdouble objx, GLdouble objy, GLdouble objz,
-                         const GLdouble modelMatrix[16],
-                         const GLdouble projMatrix[16],
-                         const GLint viewport[4],
-                         GLdouble *winx, GLdouble *winy, GLdouble *winz );
+extern GLint APIENTRY gluProject( GLdouble objx, GLdouble objy, GLdouble objz,
+                                  const GLdouble modelMatrix[16],
+                                  const GLdouble projMatrix[16],
+                                  const GLint viewport[4],
+                                  GLdouble *winx, GLdouble *winy, GLdouble *winz );
 
-extern GLint gluUnProject( GLdouble winx, GLdouble winy, GLdouble winz,
-                           const GLdouble modelMatrix[16],
-                           const GLdouble projMatrix[16],
-                           const GLint viewport[4],
-                           GLdouble *objx, GLdouble *objy, GLdouble *objz );
+extern GLint APIENTRY gluUnProject( GLdouble winx, GLdouble winy, GLdouble winz,
+                                    const GLdouble modelMatrix[16],
+                                    const GLdouble projMatrix[16],
+                                    const GLint viewport[4],
+                                    GLdouble *objx, GLdouble *objy, GLdouble *objz );
 
-extern const GLubyte* gluErrorString( GLenum errorCode );
+extern const GLubyte* APIENTRY gluErrorString( GLenum errorCode );
 
 
 
@@ -251,19 +254,19 @@ extern const GLubyte* gluErrorString( GLenum errorCode );
  *
  */
 
-extern GLint gluScaleImage( GLenum format,
-                            GLint widthin, GLint heightin,
-                            GLenum typein, const void *datain,
-                            GLint widthout, GLint heightout,
-                            GLenum typeout, void *dataout );
+extern GLint APIENTRY gluScaleImage( GLenum format,
+                                     GLint widthin, GLint heightin,
+                                     GLenum typein, const void *datain,
+                                     GLint widthout, GLint heightout,
+                                     GLenum typeout, void *dataout );
 
-extern GLint gluBuild1DMipmaps( GLenum target, GLint components,
-			        GLint width, GLenum format,
-			        GLenum type, const void *data );
+extern GLint APIENTRY gluBuild1DMipmaps( GLenum target, GLint components,
+                                         GLint width, GLenum format,
+                                         GLenum type, const void *data );
 
-extern GLint gluBuild2DMipmaps( GLenum target, GLint components,
-                                GLint width, GLint height, GLenum format,
-                                GLenum type, const void *data );
+extern GLint APIENTRY gluBuild2DMipmaps( GLenum target, GLint components,
+                                         GLint width, GLint height, GLenum format,
+                                         GLenum type, const void *data );
 
 
 
@@ -273,40 +276,40 @@ extern GLint gluBuild2DMipmaps( GLenum target, GLint components,
  *
  */
 
-extern GLUquadricObj *gluNewQuadric( void );
+extern GLUquadricObj* APIENTRY gluNewQuadric( void );
 
-extern void gluDeleteQuadric( GLUquadricObj *state );
+extern void APIENTRY gluDeleteQuadric( GLUquadricObj *state );
 
-extern void gluQuadricDrawStyle( GLUquadricObj *quadObject,
-				 GLenum drawStyle );
+extern void APIENTRY gluQuadricDrawStyle( GLUquadricObj *quadObject,
+                                          GLenum drawStyle );
 
-extern void gluQuadricOrientation( GLUquadricObj *quadObject,
-				   GLenum orientation );
+extern void APIENTRY gluQuadricOrientation( GLUquadricObj *quadObject,
+                                            GLenum orientation );
 
-extern void gluQuadricNormals( GLUquadricObj *quadObject, GLenum normals );
+extern void APIENTRY gluQuadricNormals( GLUquadricObj *quadObject, GLenum normals );
 
-extern void gluQuadricTexture( GLUquadricObj *quadObject,
-			       GLboolean textureCoords );
+extern void APIENTRY gluQuadricTexture( GLUquadricObj *quadObject,
+                                        GLboolean textureCoords );
 
-extern void gluQuadricCallback( GLUquadricObj *qobj,
-			        GLenum which, void (*fn)() );
+extern void APIENTRY gluQuadricCallback( GLUquadricObj *qobj,
+                                         GLenum which, void (*fn)() );
 
-extern void gluCylinder( GLUquadricObj *qobj,
-			 GLdouble baseRadius,
-			 GLdouble topRadius,
-			 GLdouble height,
-			 GLint slices, GLint stacks );
+extern void APIENTRY gluCylinder( GLUquadricObj *qobj,
+                                  GLdouble baseRadius,
+                                  GLdouble topRadius,
+                                  GLdouble height,
+                                  GLint slices, GLint stacks );
 
-extern void gluSphere( GLUquadricObj *qobj,
-		       GLdouble radius, GLint slices, GLint stacks );
+extern void APIENTRY gluSphere( GLUquadricObj *qobj,
+                                GLdouble radius, GLint slices, GLint stacks );
 
-extern void gluDisk( GLUquadricObj *qobj,
-		     GLdouble innerRadius, GLdouble outerRadius,
-		     GLint slices, GLint loops );
+extern void APIENTRY gluDisk( GLUquadricObj *qobj,
+                              GLdouble innerRadius, GLdouble outerRadius,
+                              GLint slices, GLint loops );
 
-extern void gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
-			    GLdouble outerRadius, GLint slices, GLint loops,
-			    GLdouble startAngle, GLdouble sweepAngle );
+extern void APIENTRY gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
+                                     GLdouble outerRadius, GLint slices, GLint loops,
+                                     GLdouble startAngle, GLdouble sweepAngle );
 
 
 
@@ -316,49 +319,49 @@ extern void gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
  *
  */
 
-extern GLUnurbsObj *gluNewNurbsRenderer( void );
+extern GLUnurbsObj* APIENTRY gluNewNurbsRenderer( void );
 
-extern void gluDeleteNurbsRenderer( GLUnurbsObj *nobj );
+extern void APIENTRY gluDeleteNurbsRenderer( GLUnurbsObj *nobj );
 
-extern void gluLoadSamplingMatrices( GLUnurbsObj *nobj,
-				     const GLfloat modelMatrix[16],
-				     const GLfloat projMatrix[16],
-				     const GLint viewport[4] );
+extern void APIENTRY gluLoadSamplingMatrices( GLUnurbsObj *nobj,
+                                              const GLfloat modelMatrix[16],
+                                              const GLfloat projMatrix[16],
+                                              const GLint viewport[4] );
 
-extern void gluNurbsProperty( GLUnurbsObj *nobj, GLenum property,
-			      GLfloat value );
+extern void APIENTRY gluNurbsProperty( GLUnurbsObj *nobj, GLenum property,
+                                       GLfloat value );
 
-extern void gluGetNurbsProperty( GLUnurbsObj *nobj, GLenum property,
-				 GLfloat *value );
+extern void APIENTRY gluGetNurbsProperty( GLUnurbsObj *nobj, GLenum property,
+                                          GLfloat *value );
 
-extern void gluBeginCurve( GLUnurbsObj *nobj );
+extern void APIENTRY gluBeginCurve( GLUnurbsObj *nobj );
 
-extern void gluEndCurve( GLUnurbsObj * nobj );
+extern void APIENTRY gluEndCurve( GLUnurbsObj * nobj );
 
-extern void gluNurbsCurve( GLUnurbsObj *nobj, GLint nknots, GLfloat *knot,
-			   GLint stride, GLfloat *ctlarray, GLint order,
-			   GLenum type );
+extern void APIENTRY gluNurbsCurve( GLUnurbsObj *nobj, GLint nknots, GLfloat *knot,
+                                    GLint stride, GLfloat *ctlarray, GLint order,
+                                    GLenum type );
 
-extern void gluBeginSurface( GLUnurbsObj *nobj );
+extern void APIENTRY gluBeginSurface( GLUnurbsObj *nobj );
 
-extern void gluEndSurface( GLUnurbsObj * nobj );
+extern void APIENTRY gluEndSurface( GLUnurbsObj * nobj );
 
-extern void gluNurbsSurface( GLUnurbsObj *nobj,
-			     GLint sknot_count, GLfloat *sknot,
-			     GLint tknot_count, GLfloat *tknot,
-			     GLint s_stride, GLint t_stride,
-			     GLfloat *ctlarray,
-			     GLint sorder, GLint torder,
-        	             GLenum type );
+extern void APIENTRY gluNurbsSurface( GLUnurbsObj *nobj,
+                                      GLint sknot_count, GLfloat *sknot,
+                                      GLint tknot_count, GLfloat *tknot,
+                                      GLint s_stride, GLint t_stride,
+                                      GLfloat *ctlarray,
+                                      GLint sorder, GLint torder,
+                                      GLenum type );
 
-extern void gluBeginTrim( GLUnurbsObj *nobj );
+extern void APIENTRY gluBeginTrim( GLUnurbsObj *nobj );
 
-extern void gluEndTrim( GLUnurbsObj *nobj );
+extern void APIENTRY gluEndTrim( GLUnurbsObj *nobj );
 
-extern void gluPwlCurve( GLUnurbsObj *nobj, GLint count, GLfloat *array,
-			 GLint stride, GLenum type );
+extern void APIENTRY gluPwlCurve( GLUnurbsObj *nobj, GLint count, GLfloat *array,
+                                  GLint stride, GLenum type );
 
-extern void gluNurbsCallback( GLUnurbsObj *nobj, GLenum which, void (*fn)() );
+extern void APIENTRY gluNurbsCallback( GLUnurbsObj *nobj, GLenum which, void (*fn)() );
 
 
 
@@ -368,21 +371,21 @@ extern void gluNurbsCallback( GLUnurbsObj *nobj, GLenum which, void (*fn)() );
  *
  */
 
-extern GLUtriangulatorObj* gluNewTess( void );
+extern GLUtriangulatorObj* APIENTRY gluNewTess( void );
 
-extern void gluTessCallback( GLUtriangulatorObj *tobj, GLenum which,
-			      void (*fn)() );
+extern void APIENTRY gluTessCallback( GLUtriangulatorObj *tobj, GLenum which,
+                                      void (*fn)() );
 
-extern void gluDeleteTess( GLUtriangulatorObj *tobj );
+extern void APIENTRY gluDeleteTess( GLUtriangulatorObj *tobj );
 
-extern void gluBeginPolygon( GLUtriangulatorObj *tobj );
+extern void APIENTRY gluBeginPolygon( GLUtriangulatorObj *tobj );
 
-extern void gluEndPolygon( GLUtriangulatorObj *tobj );
+extern void APIENTRY gluEndPolygon( GLUtriangulatorObj *tobj );
 
-extern void gluNextContour( GLUtriangulatorObj *tobj, GLenum type );
+extern void APIENTRY gluNextContour( GLUtriangulatorObj *tobj, GLenum type );
 
-extern void gluTessVertex( GLUtriangulatorObj *tobj, GLdouble v[3],
-			   void *data );
+extern void APIENTRY gluTessVertex( GLUtriangulatorObj *tobj, GLdouble v[3],
+                                    void *data );
 
 
 
@@ -392,7 +395,7 @@ extern void gluTessVertex( GLUtriangulatorObj *tobj, GLdouble v[3],
  *
  */
 
-extern const GLubyte* gluGetString( GLenum name );
+extern const GLubyte* APIENTRY gluGetString( GLenum name );
 
 
 #if defined(__BEOS__) || defined(__QUICKDRAW__)
