@@ -1,4 +1,4 @@
-/* $Id: polytest.c,v 1.4 1997/07/24 01:28:44 brianp Exp $ */
+/* $Id: polytest.c,v 1.5 1997/10/29 02:02:20 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -23,6 +23,9 @@
 
 /*
  * $Log: polytest.c,v $
+ * Revision 1.5  1997/10/29 02:02:20  brianp
+ * various MS Windows compiler changes (David Bucciarelli, v20 3dfx driver)
+ *
  * Revision 1.4  1997/07/24 01:28:44  brianp
  * changed precompiled header symbol from PCH to PC_HEADER
  *
@@ -476,7 +479,11 @@ static GLenum verify_edge_vertex_intersections(GLUtriangulatorObj *tobj)
 	return GLU_NO_ERROR;
 }
 
-static int area_compare(const void *a,const void *b)
+static int
+#if defined(FX) && defined(__WIN32__)
+_cdecl
+#endif
+area_compare(const void *a,const void *b)
 {
 	GLdouble area1,area2;
 
