@@ -1,4 +1,4 @@
-/* $Id: quadric.c,v 1.5 1997/09/17 01:51:48 brianp Exp $ */
+/* $Id: quadric.c,v 1.6 1997/10/29 02:02:20 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -23,6 +23,9 @@
 
 /*
  * $Log: quadric.c,v $
+ * Revision 1.6  1997/10/29 02:02:20  brianp
+ * various MS Windows compiler changes (David Bucciarelli, v20 3dfx driver)
+ *
  * Revision 1.5  1997/09/17 01:51:48  brianp
  * changed glu*Callback() functions to match prototype in glu.h
  *
@@ -90,7 +93,7 @@ struct GLUquadricObj {
 	GLenum Orientation;		/* GLU_INSIDE or GLU_OUTSIDE */
 	GLboolean TextureFlag;		/* Generate texture coords? */
 	GLenum Normals;		/* GLU_NONE, GLU_FLAT, or GLU_SMOOTH */
-	void (*ErrorFunc)(GLenum err);	/* Error handler callback function */
+	void (CALLBACK *ErrorFunc)(GLenum err);	/* Error handler callback function */
 };
 
 
