@@ -1,8 +1,8 @@
-/* $Id: all.h,v 1.3 1997/08/22 01:42:48 brianp Exp $ */
+/* $Id: all.h,v 1.5 1997/11/20 00:23:21 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,12 @@
 
 /*
  * $Log: all.h,v $
+ * Revision 1.5  1997/11/20 00:23:21  brianp
+ * added rect.h
+ *
+ * Revision 1.4  1997/09/27 00:16:04  brianp
+ * added colortab.h
+ *
  * Revision 1.3  1997/08/22 01:42:48  brianp
  * added api.h and hash.h
  *
@@ -72,6 +78,7 @@
 #include "bitmap.h"
 #include "blend.h"
 #include "clip.h"
+#include "colortab.h"
 #include "context.h"
 #include "config.h"
 #include "copypix.h"
@@ -102,6 +109,7 @@
 #include "polygon.h"
 #include "rastpos.h"
 #include "readpix.h"
+#include "rect.h"
 #include "scissor.h"
 #include "shade.h"
 #include "span.h"
