@@ -1,8 +1,8 @@
-/* $Id: api2.c,v 1.1 1997/08/22 01:42:18 brianp Exp $ */
+/* $Id: api2.c,v 1.5 1997/11/02 20:19:04 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,18 @@
 
 /*
  * $Log: api2.c,v $
+ * Revision 1.5  1997/11/02 20:19:04  brianp
+ * small changes to gl_TexSubImage[123]D()
+ *
+ * Revision 1.4  1997/10/29 02:24:29  brianp
+ * added glTexImage2D() hack for GLQuake (David Bucciarelli v20 3dfx)
+ *
+ * Revision 1.3  1997/10/29 01:29:09  brianp
+ * added GL_EXT_point_parameters extension from Daniel Barrero
+ *
+ * Revision 1.2  1997/09/27 00:15:05  brianp
+ * added GL_EXT_paletted_texture extension
+ *
  * Revision 1.1  1997/08/22 01:42:18  brianp
  * Initial revision
  *
@@ -1013,20 +1025,9 @@ void APIENTRY glTexImage1D( GLenum target, GLint level, GLint internalformat,
                             GLenum format, GLenum type, const GLvoid *pixels )
 {
    struct gl_image *teximage;
-   GLenum destType;
    GET_CONTEXT;
    CHECK_CONTEXT;
-   if (type==GL_UNSIGNED_BYTE) {
-      destType = GL_UNSIGNED_BYTE;
-   }
-   else if (type==GL_BITMAP) {
-      destType = GL_BITMAP;
-   }
-   else {
-      destType = GL_FLOAT;
-   }
-   teximage = gl_unpack_image( CC, width, 1, format, type,
-                               destType, pixels, GL_FALSE );
+   teximage = gl_unpack_image( CC, width, 1, format, type, pixels );
    (*CC->API.TexImage1D)( CC, target, level, internalformat,
                           width, border, format, type, teximage );
 }
@@ -1037,23 +1038,63 @@ void APIENTRY glTexImage2D( GLenum target, GLint level, GLint internalformat,
                             GLsizei width, GLsizei height, GLint border,
                             GLenum format, GLenum type, const GLvoid *pixels )
 {
-   struct gl_image *teximage;
-   GLenum destType;
-   GET_CONTEXT;
-   CHECK_CONTEXT;
-   if (type==GL_UNSIGNED_BYTE) {
-      destType = GL_UNSIGNED_BYTE;
-   }
-   else if (type==GL_BITMAP) {
-      destType = GL_BITMAP;
-   }
-   else {
-      destType = GL_FLOAT;
-   }
-   teximage = gl_unpack_image( CC, width, height, format, type,
-                               destType, pixels, GL_FALSE );
-   (*CC->API.TexImage2D)( CC, target, level, internalformat,
-                          width, height, border, format, type, teximage );
+  struct gl_image *teximage;
+#if defined(FX) && defined(__WIN32__)
+  GLvoid *newpixels=NULL;
+  GLsizei newwidth,newheight;
+  GLint x,y;
+  static GLint leveldif=0;
+  static GLuint lasttexobj=0xffffff;
+#endif
+  GET_CONTEXT;
+  CHECK_CONTEXT;
+
+#if defined(FX) && defined(__WIN32__)
+  newpixels=NULL;
+  
+  /* AN HACK for WinGLQuake*/
+  
+  if(CC->Texture.Current2D->Name!=lasttexobj) {
+    lasttexobj=CC->Texture.Current2D->Name;
+    leveldif=0;
+  }
+  
+  if((format==GL_COLOR_INDEX) && (internalformat==1))
+    internalformat=GL_COLOR_INDEX8_EXT;
+  
+  if(width>256 || height >256) {
+    while(width>256 || height >256) {
+      newwidth=width/2;
+      newheight=height/2;
+      leveldif++;
+      
+      fprintf(stderr,"Scaling: %dx%d -> %dx%d\n",width,height,newwidth,newheight);
+      
+      newpixels=malloc((newwidth+4)*newheight*4);
+      
+      for(y=0;y<newheight;y++)
+	for(x=0;x<newwidth;x++)
+	  ((GLubyte *)newpixels)[x+y*newwidth]=((GLubyte *)pixels)[x*2+y*width*2];
+      
+      if(newpixels)
+	free(pixels);
+      
+      pixels=newpixels;
+      width=newwidth;
+      height=newheight;
+    }
+    
+    level=0;
+  } else
+    level-=leveldif;
+#endif
+  teximage = gl_unpack_image( CC, width, height, format, type, pixels );
+  (*CC->API.TexImage2D)( CC, target, level, internalformat,
+			 width, height, border, format, type, teximage );
+#if defined(FX) && defined(__WIN32__)
+  if(newpixels)
+    free(newpixels);
+#endif
 }
 
 
@@ -1112,7 +1153,7 @@ void APIENTRY glTexSubImage1D( GLenum target, GLint level, GLint xoffset,
    struct gl_image *image;
    GET_CONTEXT;
    CHECK_CONTEXT;
-   image = gl_unpack_texsubimage( CC, width, 1, format, type, pixels );
+   image = gl_unpack_texsubimage( CC, width, 1, 1, format, type, pixels );
    (*CC->API.TexSubImage1D)( CC, target, level, xoffset, width,
                              format, type, image );
 }
@@ -1127,7 +1168,7 @@ void APIENTRY glTexSubImage2D( GLenum target, GLint level,
    struct gl_image *image;
    GET_CONTEXT;
    CHECK_CONTEXT;
-   image = gl_unpack_texsubimage( CC, width, height, format, type, pixels );
+   image = gl_unpack_texsubimage( CC, width, height, 1, format, type, pixels );
    (*CC->API.TexSubImage2D)( CC, target, level, xoffset, yoffset,
                              width, height, format, type, image );
 }
@@ -1510,20 +1551,9 @@ void APIENTRY glTexImage3DEXT( GLenum target, GLint level, GLenum internalformat
                                const GLvoid *pixels )
 {
    struct gl_image *teximage;
-   GLenum destType;
    GET_CONTEXT;
    CHECK_CONTEXT;
-   if (type==GL_UNSIGNED_BYTE) {
-      destType = GL_UNSIGNED_BYTE;
-   }
-   else if (type==GL_BITMAP) {
-      destType = GL_BITMAP;
-   }
-   else {
-      destType = GL_FLOAT;
-   }
-   teximage = gl_unpack_image3D( CC, width, height, depth, format, type,
-                                 destType, pixels, GL_FALSE );
+   teximage = gl_unpack_image3D( CC, width, height, depth, format, type, pixels);
    (*CC->API.TexImage3DEXT)( CC, target, level, internalformat,
                              width, height, depth, border, format, type, 
                              teximage );
@@ -1538,14 +1568,35 @@ void APIENTRY glTexSubImage3DEXT( GLenum target, GLint level, GLint xoffset,
    struct gl_image *image;
    GET_CONTEXT;
    CHECK_CONTEXT;
-   image = gl_unpack_texsubimage3D( CC, width, height, depth, format, type,
-                                    pixels );
+   image = gl_unpack_texsubimage( CC, width, height, depth, format, type, pixels );
    (*CC->API.TexSubImage3DEXT)( CC, target, level, xoffset, yoffset, zoffset,
                                 width, height, depth, format, type, image );
 }
 
 
 
+/* GL_EXT_point_parameters */
+
+void APIENTRY glPointParameterfEXT( GLenum pname, GLfloat param )
+{
+   GLfloat params[3];
+   GET_CONTEXT;
+   CHECK_CONTEXT;
+   params[0] = param;
+   params[1] = 0.0;
+   params[2] = 0.0;
+   (*CC->API.PointParameterfvEXT)(CC, pname, params);
+}
+
+
+void APIENTRY glPointParameterfvEXT( GLenum pname, const GLfloat *params )
+{
+   GET_CONTEXT;
+   CHECK_CONTEXT;
+   (*CC->API.PointParameterfvEXT)(CC, pname, params);
+}
+
+
 
 #ifdef GL_MESA_window_pos
 /*
