@@ -1,8 +1,8 @@
-/* $Id: context.c,v 1.51 1997/09/17 01:39:11 brianp Exp $ */
+/* $Id: context.c,v 1.58 1997/10/29 01:29:09 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,27 @@
 
 /*
  * $Log: context.c,v $
+ * Revision 1.58  1997/10/29 01:29:09  brianp
+ * added GL_EXT_point_parameters extension from Daniel Barrero
+ *
+ * Revision 1.57  1997/10/16 01:59:08  brianp
+ * added GL_EXT_shared_texture_palette extension
+ *
+ * Revision 1.56  1997/10/15 03:08:50  brianp
+ * don't clear depth buffer in gl_ResizeBuffersMESA()
+ *
+ * Revision 1.55  1997/09/29 22:25:28  brianp
+ * added const to a few function parameters
+ *
+ * Revision 1.54  1997/09/27 00:15:20  brianp
+ * added ctx->DirectContext flag
+ *
+ * Revision 1.53  1997/09/23 00:58:15  brianp
+ * now using hash table for texture objects
+ *
+ * Revision 1.52  1997/09/22 02:33:58  brianp
+ * display lists now implemented with hash table
+ *
  * Revision 1.51  1997/09/17 01:39:11  brianp
  * added update_clipmask() function from Michael Pichler
  *
@@ -201,6 +222,7 @@
 #include "context.h"
 #include "depth.h"
 #include "eval.h"
+#include "hash.h"
 #include "light.h"
 #include "lines.h"
 #include "dlist.h"
@@ -454,47 +476,39 @@ static void print_timings( GLcontext *ctx )
 static struct gl_shared_state *alloc_shared_state( void )
 {
    struct gl_shared_state *ss;
-   struct gl_texture_object *t1, *t2, *t3;
 
    ss = (struct gl_shared_state*) calloc( 1, sizeof(struct gl_shared_state) );
    if (!ss)
       return NULL;
 
-   /* Texture objects
-    * There are two default textures, one for the GL_TEXTURE_1D target and
-    * one for the GL_TEXTURE_2D target.  Both are named zero!
-    */
-   t1 = gl_alloc_texture_object();
-   if (!t1) {
+   ss->DisplayList = NewHashTable();
+
+   ss->TexObjects = NewHashTable();
+
+   /* Default Texture objects */
+   ss->Default1D = gl_alloc_texture_object(ss, 0, 1);
+   ss->Default2D = gl_alloc_texture_object(ss, 0, 2);
+   ss->Default3D = gl_alloc_texture_object(ss, 0, 3);
+
+   if (!ss->DisplayList || !ss->TexObjects
+       || !ss->Default1D || !ss->Default2D || !ss->Default3D) {
+      /* Ran out of memory at some point.  Free everything and return NULL */
+      if (!ss->DisplayList)
+         DeleteHashTable(ss->DisplayList);
+      if (!ss->TexObjects)
+         DeleteHashTable(ss->TexObjects);
+      if (!ss->Default1D)
+         gl_free_texture_object(ss, ss->Default1D);
+      if (!ss->Default2D)
+         gl_free_texture_object(ss, ss->Default2D);
+      if (!ss->Default3D)
+         gl_free_texture_object(ss, ss->Default3D);
       free(ss);
       return NULL;
    }
-   t1->Dimensions = 1;
-
-   t2 = gl_alloc_texture_object();
-   if (!t2) {
-      free(ss);
-      free(t1);
-      return NULL;
+   else {
+      return ss;
    }
-   t2->Dimensions = 2;
-
-   t3 = gl_alloc_texture_object();
-   if (!t3) {
-      free(ss);
-      free(t1);
-      free(t2);
-      return NULL;
-   }
-   t3->Dimensions = 3;
-
-   ss->TexObjectList = t1;
-   ss->TexObjectList->Next = t2;
-   ss->TexObjectList->Next->Next = t3;
-
-   ss->NextFreeTextureName = 1;
-
-   return ss;
 }
 
 
@@ -503,24 +517,25 @@ static struct gl_shared_state *alloc_shared_state( void )
  */
 static void free_shared_state( GLcontext *ctx, struct gl_shared_state *ss )
 {
-   /* Free the linked list of texture objects */
-   {
-      struct gl_texture_object *t, *tnext;
-      t = ss->TexObjectList;
-      while (t) {
-         tnext = t->Next;
-         gl_free_texture_object(t);
-         t = tnext;
-      }
-   }
-
    /* Free display lists */
-   {
-      int i;
-      for (i=0;i<MAX_DISPLAYLISTS;i++) {
-         gl_destroy_list( ctx, i );
+   while (1) {
+      GLuint list = HashFirstEntry(ss->DisplayList);
+      if (list) {
+         gl_destroy_list(ctx, list);
+      }
+      else {
+         break;
       }
    }
+   DeleteHashTable(ss->DisplayList);
+
+   /* Free texture objects */
+   while (ss->TexObjectList)
+   {
+      /* this function removes from linked list too! */
+      gl_free_texture_object(ss, ss->TexObjectList);
+   }
+   DeleteHashTable(ss->TexObjects);
 
    free(ss);
 }
@@ -773,6 +788,12 @@ static void initialize_context( GLcontext *ctx )
       /* Point group */
       ctx->Point.SmoothFlag = GL_FALSE;
       ctx->Point.Size = 1.0;
+      ctx->Point.Params[0] = 1.0;
+      ctx->Point.Params[1] = 0.0;
+      ctx->Point.Params[2] = 0.0;
+      ctx->Point.MinSize = 0.0;
+      ctx->Point.MaxSize = (GLfloat) MAX_POINT_SIZE;
+      ctx->Point.Threshold = 1.0;
 
       /* Polygon group */
       ctx->Polygon.CullFlag = GL_FALSE;
@@ -829,11 +850,27 @@ static void initialize_context( GLcontext *ctx )
       ASSIGN_4V( ctx->Texture.EyePlaneR, 0.0, 0.0, 0.0, 0.0 );
       ASSIGN_4V( ctx->Texture.EyePlaneQ, 0.0, 0.0, 0.0, 0.0 );
 
+#if 00
       assert( ctx->Shared->TexObjectList );
       assert( ctx->Shared->TexObjectList->Next );
       ctx->Texture.Current1D = ctx->Shared->TexObjectList;
       ctx->Texture.Current2D = ctx->Shared->TexObjectList->Next;
       ctx->Texture.Current3D = ctx->Shared->TexObjectList->Next->Next;
+#else
+      ctx->Texture.Current1D = ctx->Shared->Default1D;
+      ctx->Texture.Current2D = ctx->Shared->Default2D;
+      ctx->Texture.Current3D = ctx->Shared->Default3D;
+#endif
+
+      ctx->Texture.SharedPalette = GL_FALSE;
+      ctx->Texture.Palette[0] = 255;
+      ctx->Texture.Palette[1] = 255;
+      ctx->Texture.Palette[2] = 255;
+      ctx->Texture.Palette[3] = 255;
+      ctx->Texture.PaletteSize = 1;
+      ctx->Texture.PaletteIntFormat = GL_RGBA;
+      ctx->Texture.PaletteFormat = GL_RGBA;
+
 
       /* Transformation group */
       ctx->Transform.MatrixMode = GL_MODELVIEW;
@@ -914,6 +951,8 @@ static void initialize_context( GLcontext *ctx )
       ctx->TriangleFunc = NULL;
 #endif
 
+      ctx->DirectContext = GL_TRUE;  /* XXXX temp */
+
       ctx->CallDepth = 0;
       ctx->ExecuteFlag = GL_TRUE;
       ctx->CompileFlag = GL_FALSE;
@@ -1062,21 +1101,21 @@ static GLboolean alloc_proxy_textures( GLcontext *ctx )
    GLboolean out_of_memory;
    GLint i;
 
-   ctx->Texture.Proxy1D = gl_alloc_texture_object();
+   ctx->Texture.Proxy1D = gl_alloc_texture_object(NULL, 0, 1);
    if (!ctx->Texture.Proxy1D) {
       return GL_FALSE;
    }
 
-   ctx->Texture.Proxy2D = gl_alloc_texture_object();
+   ctx->Texture.Proxy2D = gl_alloc_texture_object(NULL, 0, 2);
    if (!ctx->Texture.Proxy2D) {
-      gl_free_texture_object(ctx->Texture.Proxy1D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy1D);
       return GL_FALSE;
    }
 
-   ctx->Texture.Proxy3D = gl_alloc_texture_object();
+   ctx->Texture.Proxy3D = gl_alloc_texture_object(NULL, 0, 3);
    if (!ctx->Texture.Proxy3D) {
-      gl_free_texture_object(ctx->Texture.Proxy1D);
-      gl_free_texture_object(ctx->Texture.Proxy2D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy1D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy2D);
       return GL_FALSE;
    }
 
@@ -1103,9 +1142,9 @@ static GLboolean alloc_proxy_textures( GLcontext *ctx )
             gl_free_texture_image(ctx->Texture.Proxy3D->Image[i]);
          }
       }
-      gl_free_texture_object(ctx->Texture.Proxy1D);
-      gl_free_texture_object(ctx->Texture.Proxy2D);
-      gl_free_texture_object(ctx->Texture.Proxy3D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy1D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy2D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy3D);
       return GL_FALSE;
    }
    else {
@@ -1212,7 +1251,6 @@ GLcontext *gl_create_context( GLvisual *visual,
 
 
 
-
 /*
  * Destroy a gl_context structure.
  */
@@ -1237,9 +1275,9 @@ void gl_destroy_context( GLcontext *ctx )
       }
 
       /* Free proxy texture objects */
-      gl_free_texture_object( ctx->Texture.Proxy1D );
-      gl_free_texture_object( ctx->Texture.Proxy2D );
-      gl_free_texture_object( ctx->Texture.Proxy3D );
+      gl_free_texture_object( NULL, ctx->Texture.Proxy1D );
+      gl_free_texture_object( NULL, ctx->Texture.Proxy2D );
+      gl_free_texture_object( NULL, ctx->Texture.Proxy3D );
 
       free( (void *) ctx );
 
@@ -1254,7 +1292,6 @@ void gl_destroy_context( GLcontext *ctx )
 
 
 
-
 /*
  * Create a new framebuffer.  A GLframebuffer is a struct which
  * encapsulates the depth, stencil and accum buffers and related
@@ -1305,8 +1342,6 @@ void gl_destroy_framebuffer( GLframebuffer *buffer )
 
 
 
-
-
 /*
  * Set the current context, binding the given frame buffer to the context.
  */
@@ -1352,7 +1387,7 @@ GLcontext *gl_get_current_context( void )
  *         dst - destination context
  *         mask - bitwise OR of GL_*_BIT flags
  */
-void gl_copy_context( GLcontext *src, GLcontext *dst, GLuint mask )
+void gl_copy_context( const GLcontext *src, GLcontext *dst, GLuint mask )
 {
    if (mask & GL_ACCUM_BUFFER_BIT) {
       MEMCPY( &dst->Accum, &src->Accum, sizeof(struct gl_accum_attrib) );
@@ -1447,7 +1482,7 @@ void gl_set_api_table( GLcontext *ctx, const struct gl_api_table *api )
  * This function is called when the Mesa user has stumbled into a code
  * path which may not be implemented fully or correctly.
  */
-void gl_problem( GLcontext *ctx, const char *s )
+void gl_problem( const GLcontext *ctx, const char *s )
 {
    fprintf( stderr, "Mesa implementation error: %s\n", s );
    fprintf( stderr, "Report to Mesa author.\n" );
@@ -1460,7 +1495,7 @@ void gl_problem( GLcontext *ctx, const char *s )
  * something illogical or if there's likely a bug in their program
  * (like enabled depth testing without a depth buffer).
  */
-void gl_warning( GLcontext *ctx, const char *s )
+void gl_warning( const GLcontext *ctx, const char *s )
 {
    GLboolean debug;
 #ifdef DEBUG
@@ -1585,7 +1620,8 @@ void gl_ResizeBuffersMESA( GLcontext *ctx )
    if (newsize && ctx->Visual->DepthBits>0) {
       /* reallocate depth buffer */
       (*ctx->Driver.AllocDepthBuffer)( ctx );
-      (*ctx->Driver.ClearDepthBuffer)( ctx );
+      /*** if scissoring enabled then clearing can cause a problem ***/
+      /***(*ctx->Driver.ClearDepthBuffer)( ctx );***/
    }
    if (newsize && ctx->Visual->StencilBits>0) {
       /* reallocate stencil buffer */
