@@ -1,8 +1,8 @@
-/* $Id: dlist.c,v 1.30 1997/07/24 01:25:01 brianp Exp $ */
+/* $Id: dlist.c,v 1.35 1997/10/29 01:29:09 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,21 @@
 
 /*
  * $Log: dlist.c,v $
+ * Revision 1.35  1997/10/29 01:29:09  brianp
+ * added GL_EXT_point_parameters extension from Daniel Barrero
+ *
+ * Revision 1.34  1997/10/02 00:48:29  brianp
+ * removed the EXEC() macro stuff, it caused bugs in gl_save_Color*()
+ *
+ * Revision 1.33  1997/09/27 00:13:44  brianp
+ * added GL_EXT_paletted_texture extension
+ *
+ * Revision 1.32  1997/09/23 00:57:52  brianp
+ * removed list>MAX_DISPLAYLISTS test
+ *
+ * Revision 1.31  1997/09/22 02:33:58  brianp
+ * display lists now implemented with hash table
+ *
  * Revision 1.30  1997/07/24 01:25:01  brianp
  * changed precompiled header symbol from PCH to PC_HEADER
  *
@@ -130,6 +145,7 @@
 #include "bitmap.h"
 #include "blend.h"
 #include "clip.h"
+#include "colortab.h"
 #include "context.h"
 #include "copypix.h"
 #include "depth.h"
@@ -138,6 +154,7 @@
 #include "eval.h"
 #include "feedback.h"
 #include "fog.h"
+#include "hash.h"
 #include "image.h"
 #include "light.h"
 #include "lines.h"
@@ -164,14 +181,6 @@
 #endif
 
 
-#if 0
-#define EXEC(FUNC)  (*ctx->Exec.FUNC)
-#else
-#define EXEC(FUNC)  gl_##FUNC
-#endif
-
-
-
 
 /*
 Functions which aren't compiled but executed immediately:
@@ -235,6 +244,8 @@ typedef enum {
 	OPCODE_COLOR_4UB,
 	OPCODE_COLOR_MASK,
 	OPCODE_COLOR_MATERIAL,
+	OPCODE_COLOR_TABLE,
+	OPCODE_COLOR_SUB_TABLE,
 	OPCODE_COPY_PIXELS,
         OPCODE_COPY_TEX_IMAGE1D,
         OPCODE_COPY_TEX_IMAGE2D,
@@ -287,7 +298,8 @@ typedef enum {
 	OPCODE_PIXEL_MAP,
 	OPCODE_PIXEL_TRANSFER,
 	OPCODE_PIXEL_ZOOM,
-	OPCODE_POINTSIZE,
+	OPCODE_POINT_SIZE,
+        OPCODE_POINT_PARAMETERS,
 	OPCODE_POLYGON_MODE,
         OPCODE_POLYGON_STIPPLE,
 	OPCODE_POLYGON_OFFSET,
@@ -422,14 +434,15 @@ static Node *make_empty_list( void )
 
 /*
  * Destroy all nodes in a display list.
- * Input:  list - list number in [0..]
+ * Input:  list - display list number
  */
 void gl_destroy_list( GLcontext *ctx, GLuint list )
 {
    Node *n, *block;
    GLboolean done;
 
-   block = n = ctx->Shared->List[list];
+   block = (Node *) HashLookup(ctx->Shared->DisplayList, list);
+   n = block;
 
    done = block ? GL_FALSE : GL_TRUE;
    while (!done) {
@@ -451,6 +464,14 @@ void gl_destroy_list( GLcontext *ctx, GLuint list )
 	    gl_free_image( (struct gl_image *) n[7].data );
 	    n += InstSize[n[0].opcode];
 	    break;
+         case OPCODE_COLOR_TABLE:
+            gl_free_image( (struct gl_image *) n[3].data );
+            n += InstSize[n[0].opcode];
+            break;
+         case OPCODE_COLOR_SUB_TABLE:
+            gl_free_image( (struct gl_image *) n[3].data );
+            n += InstSize[n[0].opcode];
+            break;
          case OPCODE_POLYGON_STIPPLE:
             free( n[1].data );
 	    n += InstSize[n[0].opcode];
@@ -493,7 +514,7 @@ void gl_destroy_list( GLcontext *ctx, GLuint list )
       }
    }
 
-   ctx->Shared->List[list] = NULL;
+   HashRemove(ctx->Shared->DisplayList, list);
 }
 
 
@@ -589,6 +610,8 @@ void gl_init_lists( void )
       InstSize[OPCODE_COLOR_4UB] = 5;
       InstSize[OPCODE_COLOR_MASK] = 5;
       InstSize[OPCODE_COLOR_MATERIAL] = 3;
+      InstSize[OPCODE_COLOR_TABLE] = 4;
+      InstSize[OPCODE_COLOR_SUB_TABLE] = 4;
       InstSize[OPCODE_COPY_PIXELS] = 6;
       InstSize[OPCODE_COPY_TEX_IMAGE1D] = 8;
       InstSize[OPCODE_COPY_TEX_IMAGE2D] = 9;
@@ -640,7 +663,8 @@ void gl_init_lists( void )
       InstSize[OPCODE_PIXEL_MAP] = 4;
       InstSize[OPCODE_PIXEL_TRANSFER] = 3;
       InstSize[OPCODE_PIXEL_ZOOM] = 3;
-      InstSize[OPCODE_POINTSIZE] = 2;
+      InstSize[OPCODE_POINT_SIZE] = 2;
+      InstSize[OPCODE_POINT_PARAMETERS] = 5;
       InstSize[OPCODE_POLYGON_MODE] = 3;
       InstSize[OPCODE_POLYGON_STIPPLE] = 2;
       InstSize[OPCODE_POLYGON_OFFSET] = 3;
@@ -709,7 +733,7 @@ void gl_save_Accum( GLcontext *ctx, GLenum op, GLfloat value )
       n[2].f = value;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Accum)( ctx, op, value );
+      (*ctx->Exec.Accum)( ctx, op, value );
    }
 }
 
@@ -722,7 +746,7 @@ void gl_save_AlphaFunc( GLcontext *ctx, GLenum func, GLclampf ref )
       n[2].f = (GLfloat) ref;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(AlphaFunc)( ctx, func, ref );
+      (*ctx->Exec.AlphaFunc)( ctx, func, ref );
    }
 }
 
@@ -734,7 +758,7 @@ void gl_save_Begin( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Begin)( ctx, mode );
+      (*ctx->Exec.Begin)( ctx, mode );
    }
 }
 
@@ -747,7 +771,7 @@ void gl_save_BindTexture( GLcontext *ctx, GLenum target, GLuint texture )
       n[2].ui = texture;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(BindTexture)( ctx, target, texture );
+      (*ctx->Exec.BindTexture)( ctx, target, texture );
    }
 }
 
@@ -769,7 +793,7 @@ void gl_save_Bitmap( GLcontext *ctx,
       n[7].data = (void *) bitmap;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Bitmap)( ctx, width, height,
+      (*ctx->Exec.Bitmap)( ctx, width, height,
                     xorig, yorig, xmove, ymove, bitmap );
    }
 }
@@ -782,7 +806,7 @@ void gl_save_BlendEquation( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(BlendEquation)( ctx, mode );
+      (*ctx->Exec.BlendEquation)( ctx, mode );
    }
 }
 
@@ -795,7 +819,7 @@ void gl_save_BlendFunc( GLcontext *ctx, GLenum sfactor, GLenum dfactor )
       n[2].e = dfactor;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(BlendFunc)( ctx, sfactor, dfactor );
+      (*ctx->Exec.BlendFunc)( ctx, sfactor, dfactor );
    }
 }
 
@@ -811,7 +835,7 @@ void gl_save_BlendColor( GLcontext *ctx, GLfloat red, GLfloat green,
       n[4].f = alpha;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(BlendColor)( ctx, red, green, blue, alpha );
+      (*ctx->Exec.BlendColor)( ctx, red, green, blue, alpha );
    }
 }
 
@@ -823,7 +847,7 @@ void gl_save_CallList( GLcontext *ctx, GLuint list )
       n[1].ui = list;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CallList)( ctx, list );
+      (*ctx->Exec.CallList)( ctx, list );
    }
 }
 
@@ -841,7 +865,7 @@ void gl_save_CallLists( GLcontext *ctx,
       }
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CallLists)( ctx, n, type, lists );
+      (*ctx->Exec.CallLists)( ctx, n, type, lists );
    }
 }
 
@@ -853,7 +877,7 @@ void gl_save_Clear( GLcontext *ctx, GLbitfield mask )
       n[1].bf = mask;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Clear)( ctx, mask );
+      (*ctx->Exec.Clear)( ctx, mask );
    }
 }
 
@@ -869,7 +893,7 @@ void gl_save_ClearAccum( GLcontext *ctx, GLfloat red, GLfloat green,
       n[4].f = alpha;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ClearAccum)( ctx, red, green, blue, alpha );
+      (*ctx->Exec.ClearAccum)( ctx, red, green, blue, alpha );
    }
 }
 
@@ -885,7 +909,7 @@ void gl_save_ClearColor( GLcontext *ctx, GLclampf red, GLclampf green,
       n[4].f = alpha;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ClearColor)( ctx, red, green, blue, alpha );
+      (*ctx->Exec.ClearColor)( ctx, red, green, blue, alpha );
    }
 }
 
@@ -897,7 +921,7 @@ void gl_save_ClearDepth( GLcontext *ctx, GLclampd depth )
       n[1].f = (GLfloat) depth;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ClearDepth)( ctx, depth );
+      (*ctx->Exec.ClearDepth)( ctx, depth );
    }
 }
 
@@ -909,7 +933,7 @@ void gl_save_ClearIndex( GLcontext *ctx, GLfloat c )
       n[1].f = c;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ClearIndex)( ctx, c );
+      (*ctx->Exec.ClearIndex)( ctx, c );
    }
 }
 
@@ -921,7 +945,7 @@ void gl_save_ClearStencil( GLcontext *ctx, GLint s )
       n[1].i = s;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ClearStencil)( ctx, s );
+      (*ctx->Exec.ClearStencil)( ctx, s );
    }
 }
 
@@ -937,7 +961,7 @@ void gl_save_ClipPlane( GLcontext *ctx, GLenum plane, const GLfloat *equ )
       n[5].f = equ[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ClipPlane)( ctx, plane, equ );
+      (*ctx->Exec.ClipPlane)( ctx, plane, equ );
    }
 }
 
@@ -951,7 +975,7 @@ void gl_save_Color3f( GLcontext *ctx, GLfloat r, GLfloat g, GLfloat b )
       n[3].f = b;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Color3f)( ctx, r, g, b );
+      (*ctx->Exec.Color3f)( ctx, r, g, b );
    }
 }
 
@@ -965,7 +989,7 @@ void gl_save_Color3fv( GLcontext *ctx, const GLfloat *c )
       n[3].f = c[2];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Color3fv)( ctx, c );
+      (*ctx->Exec.Color3fv)( ctx, c );
    }
 }
 
@@ -981,7 +1005,7 @@ void gl_save_Color4f( GLcontext *ctx, GLfloat r, GLfloat g,
       n[4].f = a;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Color4f)( ctx, r, g, b, a );
+      (*ctx->Exec.Color4f)( ctx, r, g, b, a );
    }
 }
 
@@ -996,7 +1020,7 @@ void gl_save_Color4fv( GLcontext *ctx, const GLfloat *c )
       n[4].f = c[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Color4fv)( ctx, c );
+      (*ctx->Exec.Color4fv)( ctx, c );
    }
 }
 
@@ -1012,7 +1036,7 @@ void gl_save_Color4ub( GLcontext *ctx, GLubyte r, GLubyte g,
       n[4].ub = a;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Color4ub)( ctx, r, g, b, a );
+      (*ctx->Exec.Color4ub)( ctx, r, g, b, a );
    }
 }
 
@@ -1027,7 +1051,7 @@ void gl_save_Color4ubv( GLcontext *ctx, const GLubyte *c )
       n[4].ub = c[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Color4ubv)( ctx, c );
+      (*ctx->Exec.Color4ubv)( ctx, c );
    }
 }
 
@@ -1043,7 +1067,7 @@ void gl_save_ColorMask( GLcontext *ctx, GLboolean red, GLboolean green,
       n[4].b = alpha;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ColorMask)( ctx, red, green, blue, alpha );
+      (*ctx->Exec.ColorMask)( ctx, red, green, blue, alpha );
    }
 }
 
@@ -1056,11 +1080,50 @@ void gl_save_ColorMaterial( GLcontext *ctx, GLenum face, GLenum mode )
       n[2].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ColorMaterial)( ctx, face, mode );
+      (*ctx->Exec.ColorMaterial)( ctx, face, mode );
    }
 }
 
 
+void gl_save_ColorTable( GLcontext *ctx, GLenum target, GLenum internalFormat,
+                         struct gl_image *table )
+{
+   Node *n = alloc_instruction( ctx, OPCODE_COLOR_TABLE, 3 );
+   if (n) {
+      n[1].e = target;
+      n[2].e = internalFormat;
+      n[3].data = (GLvoid *) table;
+      if (table) {
+         /* must retain this image */
+         table->RefCount = 1;
+      }
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.ColorTable)( ctx, target, internalFormat, table );
+   }
+}
+
+
+void gl_save_ColorSubTable( GLcontext *ctx, GLenum target,
+                            GLsizei start, struct gl_image *data )
+{
+   Node *n = alloc_instruction( ctx, OPCODE_COLOR_SUB_TABLE, 3 );
+   if (n) {
+      n[1].e = target;
+      n[2].i = start;
+      n[3].data = (GLvoid *) data;
+      if (data) {
+         /* must retain this image */
+         data->RefCount = 1;
+      }
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.ColorSubTable)( ctx, target, start, data );
+   }
+}
+
+
+
 void gl_save_CopyPixels( GLcontext *ctx, GLint x, GLint y,
 			 GLsizei width, GLsizei height, GLenum type )
 {
@@ -1073,7 +1136,7 @@ void gl_save_CopyPixels( GLcontext *ctx, GLint x, GLint y,
       n[5].e = type;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CopyPixels)( ctx, x, y, width, height, type );
+      (*ctx->Exec.CopyPixels)( ctx, x, y, width, height, type );
    }
 }
 
@@ -1096,7 +1159,7 @@ void gl_save_CopyTexImage1D( GLcontext *ctx,
       n[7].i = border;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CopyTexImage1D)( ctx, target, level, internalformat,
+      (*ctx->Exec.CopyTexImage1D)( ctx, target, level, internalformat,
                             x, y, width, border );
    }
 }
@@ -1120,7 +1183,7 @@ void gl_save_CopyTexImage2D( GLcontext *ctx,
       n[8].i = border;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CopyTexImage2D)( ctx, target, level, internalformat,
+      (*ctx->Exec.CopyTexImage2D)( ctx, target, level, internalformat,
                             x, y, width, height, border );
    }
 }
@@ -1142,7 +1205,7 @@ void gl_save_CopyTexSubImage1D( GLcontext *ctx,
       n[6].i = width;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CopyTexSubImage1D)( ctx, target, level, xoffset, x, y, width );
+      (*ctx->Exec.CopyTexSubImage1D)( ctx, target, level, xoffset, x, y, width );
    }
 }
 
@@ -1165,7 +1228,7 @@ void gl_save_CopyTexSubImage2D( GLcontext *ctx,
       n[8].i = height;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CopyTexSubImage2D)( ctx, target, level, xoffset, yoffset,
+      (*ctx->Exec.CopyTexSubImage2D)( ctx, target, level, xoffset, yoffset,
                                x, y, width, height );
    }
 }
@@ -1190,7 +1253,7 @@ void gl_save_CopyTexSubImage3DEXT( GLcontext *ctx,
       n[9].i = height;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CopyTexSubImage3DEXT)( ctx, target, level, xoffset, yoffset, zoffset,
+      (*ctx->Exec.CopyTexSubImage3DEXT)( ctx, target, level, xoffset, yoffset, zoffset,
                                x, y, width, height );
    }
 }
@@ -1203,7 +1266,7 @@ void gl_save_CullFace( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(CullFace)( ctx, mode );
+      (*ctx->Exec.CullFace)( ctx, mode );
    }
 }
 
@@ -1215,7 +1278,7 @@ void gl_save_DepthFunc( GLcontext *ctx, GLenum func )
       n[1].e = func;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(DepthFunc)( ctx, func );
+      (*ctx->Exec.DepthFunc)( ctx, func );
    }
 }
 
@@ -1227,7 +1290,7 @@ void gl_save_DepthMask( GLcontext *ctx, GLboolean mask )
       n[1].b = mask;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(DepthMask)( ctx, mask );
+      (*ctx->Exec.DepthMask)( ctx, mask );
    }
 }
 
@@ -1240,7 +1303,7 @@ void gl_save_DepthRange( GLcontext *ctx, GLclampd nearval, GLclampd farval )
       n[2].f = (GLfloat) farval;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(DepthRange)( ctx, nearval, farval );
+      (*ctx->Exec.DepthRange)( ctx, nearval, farval );
    }
 }
 
@@ -1252,7 +1315,7 @@ void gl_save_Disable( GLcontext *ctx, GLenum cap )
       n[1].e = cap;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Disable)( ctx, cap );
+      (*ctx->Exec.Disable)( ctx, cap );
    }
 }
 
@@ -1264,7 +1327,7 @@ void gl_save_DrawBuffer( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(DrawBuffer)( ctx, mode );
+      (*ctx->Exec.DrawBuffer)( ctx, mode );
    }
 }
 
@@ -1282,7 +1345,7 @@ void gl_save_DrawPixels( GLcontext *ctx, GLsizei width, GLsizei height,
    }
 /* Special case:  gl_DrawPixels takes care of GL_COMPILE_AND_EXECUTE case!
    if (ctx->ExecuteFlag) {
-      EXEC(DrawPixels)( ctx, width, height, format, type, pixels );
+      (*ctx->Exec.DrawPixels)( ctx, width, height, format, type, pixels );
    }
 */
 }
@@ -1295,7 +1358,7 @@ void gl_save_EdgeFlag( GLcontext *ctx, GLboolean flag )
       n[1].b = flag;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(EdgeFlag)( ctx, flag );
+      (*ctx->Exec.EdgeFlag)( ctx, flag );
    }
 }
 
@@ -1307,7 +1370,7 @@ void gl_save_Enable( GLcontext *ctx, GLenum cap )
       n[1].e = cap;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Enable)( ctx, cap );
+      (*ctx->Exec.Enable)( ctx, cap );
    }
 }
 
@@ -1316,7 +1379,7 @@ void gl_save_End( GLcontext *ctx )
 {
    (void) alloc_instruction( ctx, OPCODE_END, 0 );
    if (ctx->ExecuteFlag) {
-      EXEC(End)( ctx );
+      (*ctx->Exec.End)( ctx );
    }
 }
 
@@ -1328,7 +1391,7 @@ void gl_save_EvalCoord1f( GLcontext *ctx, GLfloat u )
       n[1].f = u;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(EvalCoord1f)( ctx, u );
+      (*ctx->Exec.EvalCoord1f)( ctx, u );
    }
 }
 
@@ -1341,7 +1404,7 @@ void gl_save_EvalCoord2f( GLcontext *ctx, GLfloat u, GLfloat v )
       n[2].f = v;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(EvalCoord2f)( ctx, u, v );
+      (*ctx->Exec.EvalCoord2f)( ctx, u, v );
    }
 }
 
@@ -1356,7 +1419,7 @@ void gl_save_EvalMesh1( GLcontext *ctx,
       n[3].i = i2;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(EvalMesh1)( ctx, mode, i1, i2 );
+      (*ctx->Exec.EvalMesh1)( ctx, mode, i1, i2 );
    }
 }
 
@@ -1373,7 +1436,7 @@ void gl_save_EvalMesh2( GLcontext *ctx,
       n[5].i = j2;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(EvalMesh2)( ctx, mode, i1, i2, j1, j2 );
+      (*ctx->Exec.EvalMesh2)( ctx, mode, i1, i2, j1, j2 );
    }
 }
 
@@ -1385,7 +1448,7 @@ void gl_save_EvalPoint1( GLcontext *ctx, GLint i )
       n[1].i = i;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(EvalPoint1)( ctx, i );
+      (*ctx->Exec.EvalPoint1)( ctx, i );
    }
 }
 
@@ -1398,7 +1461,7 @@ void gl_save_EvalPoint2( GLcontext *ctx, GLint i, GLint j )
       n[2].i = j;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(EvalPoint2)( ctx, i, j );
+      (*ctx->Exec.EvalPoint2)( ctx, i, j );
    }
 }
 
@@ -1414,7 +1477,7 @@ void gl_save_Fogfv( GLcontext *ctx, GLenum pname, const GLfloat *params )
       n[5].f = params[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Fogfv)( ctx, pname, params );
+      (*ctx->Exec.Fogfv)( ctx, pname, params );
    }
 }
 
@@ -1426,7 +1489,7 @@ void gl_save_FrontFace( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(FrontFace)( ctx, mode );
+      (*ctx->Exec.FrontFace)( ctx, mode );
    }
 }
 
@@ -1445,7 +1508,7 @@ void gl_save_Frustum( GLcontext *ctx, GLdouble left, GLdouble right,
       n[6].f = farval;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Frustum)( ctx, left, right, bottom, top, nearval, farval );
+      (*ctx->Exec.Frustum)( ctx, left, right, bottom, top, nearval, farval );
    }
 }
 
@@ -1458,7 +1521,7 @@ void gl_save_Hint( GLcontext *ctx, GLenum target, GLenum mode )
       n[2].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Hint)( ctx, target, mode );
+      (*ctx->Exec.Hint)( ctx, target, mode );
    }
 }
 
@@ -1470,7 +1533,7 @@ void gl_save_Indexi( GLcontext *ctx, GLint index )
       n[1].i = index;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Indexi)( ctx, index );
+      (*ctx->Exec.Indexi)( ctx, index );
    }
 }
 
@@ -1482,7 +1545,7 @@ void gl_save_Indexf( GLcontext *ctx, GLfloat index )
       n[1].i = (GLint) index;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Indexf)( ctx,index );
+      (*ctx->Exec.Indexf)( ctx,index );
    }
 }
 
@@ -1494,7 +1557,7 @@ void gl_save_IndexMask( GLcontext *ctx, GLuint mask )
       n[1].ui = mask;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(IndexMask)( ctx, mask );
+      (*ctx->Exec.IndexMask)( ctx, mask );
    }
 }
 
@@ -1503,7 +1566,7 @@ void gl_save_InitNames( GLcontext *ctx )
 {
    (void) alloc_instruction( ctx, OPCODE_INIT_NAMES, 0 );
    if (ctx->ExecuteFlag) {
-      EXEC(InitNames)( ctx );
+      (*ctx->Exec.InitNames)( ctx );
    }
 }
 
@@ -1521,7 +1584,7 @@ void gl_save_Lightfv( GLcontext *ctx, GLenum light, GLenum pname,
       }
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Lightfv)( ctx, light, pname, params, numparams );
+      (*ctx->Exec.Lightfv)( ctx, light, pname, params, numparams );
    }
 }
 
@@ -1538,7 +1601,7 @@ void gl_save_LightModelfv( GLcontext *ctx,
       n[5].f = params[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(LightModelfv)( ctx, pname, params );
+      (*ctx->Exec.LightModelfv)( ctx, pname, params );
    }
 }
 
@@ -1551,7 +1614,7 @@ void gl_save_LineStipple( GLcontext *ctx, GLint factor, GLushort pattern )
       n[2].us = pattern;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(LineStipple)( ctx, factor, pattern );
+      (*ctx->Exec.LineStipple)( ctx, factor, pattern );
    }
 }
 
@@ -1563,7 +1626,7 @@ void gl_save_LineWidth( GLcontext *ctx, GLfloat width )
       n[1].f = width;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(LineWidth)( ctx, width );
+      (*ctx->Exec.LineWidth)( ctx, width );
    }
 }
 
@@ -1575,7 +1638,7 @@ void gl_save_ListBase( GLcontext *ctx, GLuint base )
       n[1].ui = base;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ListBase)( ctx, base );
+      (*ctx->Exec.ListBase)( ctx, base );
    }
 }
 
@@ -1584,7 +1647,7 @@ void gl_save_LoadIdentity( GLcontext *ctx )
 {
    (void) alloc_instruction( ctx, OPCODE_LOAD_IDENTITY, 0 );
    if (ctx->ExecuteFlag) {
-      EXEC(LoadIdentity)( ctx );
+      (*ctx->Exec.LoadIdentity)( ctx );
    }
 }
 
@@ -1599,7 +1662,7 @@ void gl_save_LoadMatrixf( GLcontext *ctx, const GLfloat *m )
       }
    }
    if (ctx->ExecuteFlag) {
-      EXEC(LoadMatrixf)( ctx, m );
+      (*ctx->Exec.LoadMatrixf)( ctx, m );
    }
 }
 
@@ -1611,7 +1674,7 @@ void gl_save_LoadName( GLcontext *ctx, GLuint name )
       n[1].ui = name;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(LoadName)( ctx, name );
+      (*ctx->Exec.LoadName)( ctx, name );
    }
 }
 
@@ -1623,7 +1686,7 @@ void gl_save_LogicOp( GLcontext *ctx, GLenum opcode )
       n[1].e = opcode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(LogicOp)( ctx, opcode );
+      (*ctx->Exec.LogicOp)( ctx, opcode );
    }
 }
 
@@ -1642,7 +1705,7 @@ void gl_save_Map1f( GLcontext *ctx,
       n[6].data = (void *) points;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Map1f)( ctx, target, u1, u2, stride, order, points, GL_TRUE );
+      (*ctx->Exec.Map1f)( ctx, target, u1, u2, stride, order, points, GL_TRUE );
    }
 }
 
@@ -1666,7 +1729,7 @@ void gl_save_Map2f( GLcontext *ctx, GLenum target,
       n[10].data = (void *) points;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Map2f)( ctx, target,
+      (*ctx->Exec.Map2f)( ctx, target,
                         u1, u2, ustride, uorder,
                         v1, v2, vstride, vorder, points, GL_TRUE );
    }
@@ -1682,7 +1745,7 @@ void gl_save_MapGrid1f( GLcontext *ctx, GLint un, GLfloat u1, GLfloat u2 )
       n[3].f = u2;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(MapGrid1f)( ctx, un, u1, u2 );
+      (*ctx->Exec.MapGrid1f)( ctx, un, u1, u2 );
    }
 }
 
@@ -1701,7 +1764,7 @@ void gl_save_MapGrid2f( GLcontext *ctx,
       n[6].f = v2;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(MapGrid2f)( ctx, un, u1, u2, vn, v1, v2 );
+      (*ctx->Exec.MapGrid2f)( ctx, un, u1, u2, vn, v1, v2 );
    }
 }
 
@@ -1719,7 +1782,7 @@ void gl_save_Materialfv( GLcontext *ctx,
       n[6].f = params[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Materialfv)( ctx, face, pname, params );
+      (*ctx->Exec.Materialfv)( ctx, face, pname, params );
    }
 }
 
@@ -1731,7 +1794,7 @@ void gl_save_MatrixMode( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(MatrixMode)( ctx, mode );
+      (*ctx->Exec.MatrixMode)( ctx, mode );
    }
 }
 
@@ -1746,7 +1809,7 @@ void gl_save_MultMatrixf( GLcontext *ctx, const GLfloat *m )
       }
    }
    if (ctx->ExecuteFlag) {
-      EXEC(MultMatrixf)( ctx, m );
+      (*ctx->Exec.MultMatrixf)( ctx, m );
    }
 }
 
@@ -1767,7 +1830,7 @@ void gl_save_Normal3fv( GLcontext *ctx, const GLfloat norm[3] )
       n[3].f = norm[2];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Normal3fv)( ctx, norm );
+      (*ctx->Exec.Normal3fv)( ctx, norm );
    }
 }
 
@@ -1781,7 +1844,7 @@ void gl_save_Normal3f( GLcontext *ctx, GLfloat nx, GLfloat ny, GLfloat nz )
       n[3].f = nz;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Normal3f)( ctx, nx, ny, nz );
+      (*ctx->Exec.Normal3f)( ctx, nx, ny, nz );
    }
 }
 
@@ -1800,7 +1863,7 @@ void gl_save_Ortho( GLcontext *ctx, GLdouble left, GLdouble right,
       n[6].f = farval;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Ortho)( ctx, left, right, bottom, top, nearval, farval );
+      (*ctx->Exec.Ortho)( ctx, left, right, bottom, top, nearval, farval );
    }
 }
 
@@ -1816,7 +1879,7 @@ void gl_save_PixelMapfv( GLcontext *ctx,
       MEMCPY( n[3].data, (void *) values, mapsize * sizeof(GLfloat) );
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PixelMapfv)( ctx, map, mapsize, values );
+      (*ctx->Exec.PixelMapfv)( ctx, map, mapsize, values );
    }
 }
 
@@ -1829,7 +1892,7 @@ void gl_save_PixelTransferf( GLcontext *ctx, GLenum pname, GLfloat param )
       n[2].f = param;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PixelTransferf)( ctx, pname, param );
+      (*ctx->Exec.PixelTransferf)( ctx, pname, param );
    }
 }
 
@@ -1842,19 +1905,35 @@ void gl_save_PixelZoom( GLcontext *ctx, GLfloat xfactor, GLfloat yfactor )
       n[2].f = yfactor;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PixelZoom)( ctx, xfactor, yfactor );
+      (*ctx->Exec.PixelZoom)( ctx, xfactor, yfactor );
+   }
+}
+
+
+void gl_save_PointParameterfvEXT( GLcontext *ctx, GLenum pname,
+                                  const GLfloat *params)
+{
+   Node *n = alloc_instruction( ctx, OPCODE_POINT_PARAMETERS, 4 );
+   if (n) {
+      n[1].e = pname;
+      n[2].f = params[0];
+      n[3].f = params[1];
+      n[4].f = params[2];
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.PointParameterfvEXT)( ctx, pname, params );
    }
 }
 
 
 void gl_save_PointSize( GLcontext *ctx, GLfloat size )
 {
-   Node *n = alloc_instruction( ctx, OPCODE_POINTSIZE, 1 );
+   Node *n = alloc_instruction( ctx, OPCODE_POINT_SIZE, 1 );
    if (n) {
       n[1].f = size;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PointSize)( ctx, size );
+      (*ctx->Exec.PointSize)( ctx, size );
    }
 }
 
@@ -1867,7 +1946,7 @@ void gl_save_PolygonMode( GLcontext *ctx, GLenum face, GLenum mode )
       n[2].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PolygonMode)( ctx, face, mode );
+      (*ctx->Exec.PolygonMode)( ctx, face, mode );
    }
 }
 
@@ -1882,7 +1961,7 @@ void gl_save_PolygonStipple( GLcontext *ctx, const GLubyte *mask )
       MEMCPY( data, mask, 32 * 4 );
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PolygonStipple)( ctx, mask );
+      (*ctx->Exec.PolygonStipple)( ctx, mask );
    }
 }
 
@@ -1895,7 +1974,7 @@ void gl_save_PolygonOffset( GLcontext *ctx, GLfloat factor, GLfloat units )
       n[2].f = units;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PolygonOffset)( ctx, factor, units );
+      (*ctx->Exec.PolygonOffset)( ctx, factor, units );
    }
 }
 
@@ -1904,7 +1983,7 @@ void gl_save_PopAttrib( GLcontext *ctx )
 {
    (void) alloc_instruction( ctx, OPCODE_POP_ATTRIB, 0 );
    if (ctx->ExecuteFlag) {
-      EXEC(PopAttrib)( ctx );
+      (*ctx->Exec.PopAttrib)( ctx );
    }
 }
 
@@ -1913,7 +1992,7 @@ void gl_save_PopMatrix( GLcontext *ctx )
 {
    (void) alloc_instruction( ctx, OPCODE_POP_MATRIX, 0 );
    if (ctx->ExecuteFlag) {
-      EXEC(PopMatrix)( ctx );
+      (*ctx->Exec.PopMatrix)( ctx );
    }
 }
 
@@ -1922,7 +2001,7 @@ void gl_save_PopName( GLcontext *ctx )
 {
    (void) alloc_instruction( ctx, OPCODE_POP_NAME, 0 );
    if (ctx->ExecuteFlag) {
-      EXEC(PopName)( ctx );
+      (*ctx->Exec.PopName)( ctx );
    }
 }
 
@@ -1941,7 +2020,7 @@ void gl_save_PrioritizeTextures( GLcontext *ctx,
       }
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PrioritizeTextures)( ctx, num, textures, priorities );
+      (*ctx->Exec.PrioritizeTextures)( ctx, num, textures, priorities );
    }
 }
 
@@ -1953,7 +2032,7 @@ void gl_save_PushAttrib( GLcontext *ctx, GLbitfield mask )
       n[1].bf = mask;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PushAttrib)( ctx, mask );
+      (*ctx->Exec.PushAttrib)( ctx, mask );
    }
 }
 
@@ -1962,7 +2041,7 @@ void gl_save_PushMatrix( GLcontext *ctx )
 {
    (void) alloc_instruction( ctx, OPCODE_PUSH_MATRIX, 0 );
    if (ctx->ExecuteFlag) {
-      EXEC(PushMatrix)( ctx );
+      (*ctx->Exec.PushMatrix)( ctx );
    }
 }
 
@@ -1974,7 +2053,7 @@ void gl_save_PushName( GLcontext *ctx, GLuint name )
       n[1].ui = name;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PushName)( ctx, name );
+      (*ctx->Exec.PushName)( ctx, name );
    }
 }
 
@@ -1990,7 +2069,7 @@ void gl_save_RasterPos4f( GLcontext *ctx,
       n[4].f = w;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(RasterPos4f)( ctx, x, y, z, w );
+      (*ctx->Exec.RasterPos4f)( ctx, x, y, z, w );
    }
 }
 
@@ -2002,7 +2081,7 @@ void gl_save_PassThrough( GLcontext *ctx, GLfloat token )
       n[1].f = token;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(PassThrough)( ctx, token );
+      (*ctx->Exec.PassThrough)( ctx, token );
    }
 }
 
@@ -2014,7 +2093,7 @@ void gl_save_ReadBuffer( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ReadBuffer)( ctx, mode );
+      (*ctx->Exec.ReadBuffer)( ctx, mode );
    }
 }
 
@@ -2030,7 +2109,7 @@ void gl_save_Rectf( GLcontext *ctx,
       n[4].f = y2;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Rectf)( ctx, x1, y1, x2, y2 );
+      (*ctx->Exec.Rectf)( ctx, x1, y1, x2, y2 );
    }
 }
 
@@ -2053,7 +2132,7 @@ void gl_save_Scalef( GLcontext *ctx, GLfloat x, GLfloat y, GLfloat z )
       n[3].f = z;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Scalef)( ctx, x, y, z );
+      (*ctx->Exec.Scalef)( ctx, x, y, z );
    }
 }
 
@@ -2069,7 +2148,7 @@ void gl_save_Scissor( GLcontext *ctx,
       n[4].i = height;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Scissor)( ctx, x, y, width, height );
+      (*ctx->Exec.Scissor)( ctx, x, y, width, height );
    }
 }
 
@@ -2081,7 +2160,7 @@ void gl_save_ShadeModel( GLcontext *ctx, GLenum mode )
       n[1].e = mode;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(ShadeModel)( ctx, mode );
+      (*ctx->Exec.ShadeModel)( ctx, mode );
    }
 }
 
@@ -2095,7 +2174,7 @@ void gl_save_StencilFunc( GLcontext *ctx, GLenum func, GLint ref, GLuint mask )
       n[3].ui = mask;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(StencilFunc)( ctx, func, ref, mask );
+      (*ctx->Exec.StencilFunc)( ctx, func, ref, mask );
    }
 }
 
@@ -2107,7 +2186,7 @@ void gl_save_StencilMask( GLcontext *ctx, GLuint mask )
       n[1].ui = mask;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(StencilMask)( ctx, mask );
+      (*ctx->Exec.StencilMask)( ctx, mask );
    }
 }
 
@@ -2122,7 +2201,7 @@ void gl_save_StencilOp( GLcontext *ctx,
       n[3].e = zpass;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(StencilOp)( ctx, fail, zfail, zpass );
+      (*ctx->Exec.StencilOp)( ctx, fail, zfail, zpass );
    }
 }
 
@@ -2135,7 +2214,7 @@ void gl_save_TexCoord2f( GLcontext *ctx, GLfloat s, GLfloat t )
       n[2].f = t;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexCoord2f)( ctx, s, t );
+      (*ctx->Exec.TexCoord2f)( ctx, s, t );
    }
 }
 
@@ -2151,7 +2230,7 @@ void gl_save_TexCoord4f( GLcontext *ctx, GLfloat s, GLfloat t,
       n[4].f = q;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexCoord4f)( ctx, s, t, r, q );
+      (*ctx->Exec.TexCoord4f)( ctx, s, t, r, q );
    }
 }
 
@@ -2169,7 +2248,7 @@ void gl_save_TexEnvfv( GLcontext *ctx,
       n[6].f = params[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexEnvfv)( ctx, target, pname, params );
+      (*ctx->Exec.TexEnvfv)( ctx, target, pname, params );
    }
 }
 
@@ -2187,7 +2266,7 @@ void gl_save_TexGenfv( GLcontext *ctx,
       n[6].f = params[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexGenfv)( ctx, coord, pname, params );
+      (*ctx->Exec.TexGenfv)( ctx, coord, pname, params );
    }
 }
 
@@ -2205,7 +2284,7 @@ void gl_save_TexParameterfv( GLcontext *ctx, GLenum target,
       n[6].f = params[3];
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexParameterfv)( ctx, target, pname, params );
+      (*ctx->Exec.TexParameterfv)( ctx, target, pname, params );
    }
 }
 
@@ -2232,7 +2311,7 @@ void gl_save_TexImage1D( GLcontext *ctx, GLenum target,
       }
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexImage1D)( ctx, target, level, components, width,
+      (*ctx->Exec.TexImage1D)( ctx, target, level, components, width,
                         border, format, type, teximage );
    }
 }
@@ -2261,7 +2340,7 @@ void gl_save_TexImage2D( GLcontext *ctx, GLenum target,
       }
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexImage2D)( ctx, target, level, components, width,
+      (*ctx->Exec.TexImage2D)( ctx, target, level, components, width,
                         height, border, format, type, teximage );
    }
 }
@@ -2288,7 +2367,7 @@ void gl_save_TexImage3DEXT( GLcontext *ctx, GLenum target,
       n[10].data = teximage;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexImage3DEXT)( ctx, target, level, components, width,
+      (*ctx->Exec.TexImage3DEXT)( ctx, target, level, components, width,
                            height, depth, border, format, type, teximage );
    }
 }
@@ -2312,7 +2391,7 @@ void gl_save_TexSubImage1D( GLcontext *ctx,
          image->RefCount = 1;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexSubImage1D)( ctx, target, level, xoffset, width,
+      (*ctx->Exec.TexSubImage1D)( ctx, target, level, xoffset, width,
                            format, type, image );
    }
 }
@@ -2340,7 +2419,7 @@ void gl_save_TexSubImage2D( GLcontext *ctx,
          image->RefCount = 1;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexSubImage2D)( ctx, target, level, xoffset, yoffset,
+      (*ctx->Exec.TexSubImage2D)( ctx, target, level, xoffset, yoffset,
                            width, height, format, type, image );
    }
 }
@@ -2370,7 +2449,7 @@ void gl_save_TexSubImage3DEXT( GLcontext *ctx,
          image->RefCount = 1;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(TexSubImage3DEXT)( ctx, target, level, xoffset, yoffset, zoffset,
+      (*ctx->Exec.TexSubImage3DEXT)( ctx, target, level, xoffset, yoffset, zoffset,
                               width, height, depth, format, type, image );
    }
 }
@@ -2385,7 +2464,7 @@ void gl_save_Translatef( GLcontext *ctx, GLfloat x, GLfloat y, GLfloat z )
       n[3].f = z;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Translatef)( ctx, x, y, z );
+      (*ctx->Exec.Translatef)( ctx, x, y, z );
    }
 }
 
@@ -2458,7 +2537,7 @@ void gl_save_Viewport( GLcontext *ctx,
       n[4].i = (GLint) height;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(Viewport)( ctx, x, y, width, height );
+      (*ctx->Exec.Viewport)( ctx, x, y, width, height );
    }
 }
 
@@ -2474,7 +2553,7 @@ void gl_save_WindowPos4fMESA( GLcontext *ctx,
       n[4].f = w;
    }
    if (ctx->ExecuteFlag) {
-      EXEC(WindowPos4fMESA)( ctx, x, y, z, w );
+      (*ctx->Exec.WindowPos4fMESA)( ctx, x, y, z, w );
    }
 }
 
@@ -2489,7 +2568,7 @@ void gl_save_WindowPos4fMESA( GLcontext *ctx,
  * Execute a display list.  Note that the ListBase offset must have already
  * been added before calling this function.  I.e. the list argument is
  * the absolute list number, not relative to ListBase.
- * Input:  list - list number in [1..]
+ * Input:  list - display list number
  */
 static void execute_list( GLcontext *ctx, GLuint list )
 {
@@ -2502,7 +2581,8 @@ static void execute_list( GLcontext *ctx, GLuint list )
 
    ctx->CallDepth++;
 
-   n = ctx->Shared->List[list-1];
+   n = (Node *) HashLookup(ctx->Shared->DisplayList, list);
+
    done = GL_FALSE;
    while (!done) {
       opcode = n[0].opcode;
@@ -2634,6 +2714,13 @@ static void execute_list( GLcontext *ctx, GLuint list )
 	 case OPCODE_COLOR_MATERIAL:
 	    gl_ColorMaterial( ctx, n[1].e, n[2].e );
 	    break;
+         case OPCODE_COLOR_TABLE:
+            gl_ColorTable( ctx, n[1].e, n[2].e, (struct gl_image *) n[3].data);
+            break;
+         case OPCODE_COLOR_SUB_TABLE:
+            gl_ColorSubTable( ctx, n[1].e, n[2].i,
+                              (struct gl_image *) n[3].data);
+            break;
 	 case OPCODE_COPY_PIXELS:
 	    gl_CopyPixels( ctx, n[1].i, n[2].i,
 			   (GLsizei) n[3].i, (GLsizei) n[4].i, n[5].e );
@@ -2840,9 +2927,18 @@ static void execute_list( GLcontext *ctx, GLuint list )
 	 case OPCODE_PIXEL_ZOOM:
 	    gl_PixelZoom( ctx, n[1].f, n[2].f );
 	    break;
-	 case OPCODE_POINTSIZE:
+	 case OPCODE_POINT_SIZE:
 	    gl_PointSize( ctx, n[1].f );
 	    break;
+	 case OPCODE_POINT_PARAMETERS:
+	    {
+		GLfloat params[3];
+		params[0] = n[2].f;
+		params[1] = n[3].f;
+		params[2] = n[4].f;
+		gl_PointParameterfvEXT( ctx, n[1].e, params ); 
+	    }
+	    break;
 	 case OPCODE_POLYGON_MODE:
 	    gl_PolygonMode( ctx, n[1].e, n[2].e );
 	    break;
@@ -3027,7 +3123,7 @@ static void execute_list( GLcontext *ctx, GLuint list )
  */
 GLboolean gl_IsList( GLcontext *ctx, GLuint list )
 {
-   if (list>0 && list<=MAX_DISPLAYLISTS && ctx->Shared->List[list-1]) {
+   if (list > 0 && HashLookup(ctx->Shared->DisplayList, list)) {
       return GL_TRUE;
    }
    else {
@@ -3053,10 +3149,7 @@ void gl_DeleteLists( GLcontext *ctx, GLuint list, GLsizei range )
       return;
    }
    for (i=list;i<list+range;i++) {
-      if (i<=MAX_DISPLAYLISTS && ctx->Shared->List[i-1]) {
-         gl_destroy_list( ctx, i-1 );
-         ctx->Shared->List[i-1] = NULL;
-      }
+      gl_destroy_list( ctx, i );
    }
 }
 
@@ -3068,7 +3161,7 @@ void gl_DeleteLists( GLcontext *ctx, GLuint list, GLsizei range )
  */
 GLuint gl_GenLists( GLcontext *ctx, GLsizei range )
 {
-   GLuint i, freecount;
+   GLuint base;
 
    if (INSIDE_BEGIN_END(ctx)) {
       gl_error( ctx, GL_INVALID_OPERATION, "glGenLists" );
@@ -3082,27 +3175,15 @@ GLuint gl_GenLists( GLcontext *ctx, GLsizei range )
       return 0;
    }
 
-   i = 0;
-   freecount = 0;
-   for (i=0; i<MAX_DISPLAYLISTS; i++ ) {
-      if (ctx->Shared->List[i]==NULL) {
-         freecount++;
-         if (freecount==range) {
-	    /* We found 'range' consecutive free lists. */
-            /* Now reserve the IDs with empty display lists. */
-	    GLuint k;
-	    GLuint n = i-range+2;
-	    for (k=n;k<n+range;k++) {
-	       ctx->Shared->List[k-1] = make_empty_list();
-	    }
-            return n;
-         }
-      }
-      else {
-         freecount = 0;
+   base = HashFindFreeKeyBlock(ctx->Shared->DisplayList, range);
+   if (base) {
+      /* reserve the list IDs by with empty/dummy lists */
+      GLuint i;
+      for (i=0; i<range; i++) {
+         HashInsert(ctx->Shared->DisplayList, base+i, make_empty_list());
       }
    }
-   return 0;
+   return base;
 }
 
 
@@ -3116,7 +3197,7 @@ void gl_NewList( GLcontext *ctx, GLuint list, GLenum mode )
       gl_error( ctx, GL_INVALID_OPERATION, "glNewList" );
       return;
    }
-   if (list==0 || list>MAX_DISPLAYLISTS) {
+   if (list==0) {
       gl_error( ctx, GL_INVALID_VALUE, "glNewList" );
       return;
    }
@@ -3164,11 +3245,10 @@ void gl_EndList( GLcontext *ctx )
 
    n = alloc_instruction( ctx, OPCODE_END_OF_LIST, 0 );
 
+   /* Destroy old list, if any */
+   gl_destroy_list(ctx, CurrentListNum);
    /* Install the list */
-   if (ctx->Shared->List[CurrentListNum-1]) {
-      gl_destroy_list( ctx, CurrentListNum-1 );
-   }
-   ctx->Shared->List[CurrentListNum-1] = CurrentListPtr;
+   HashInsert(ctx->Shared->DisplayList, CurrentListNum, CurrentListPtr);
 
    CurrentListNum = 0;
    CurrentListPtr = NULL;
@@ -3282,11 +3362,11 @@ static void print_list( GLcontext *ctx, FILE *f, GLuint list )
       return;
    }
 
-   n = ctx->Shared->List[list-1];
+   n = (Node *) HashLookup(ctx->Shared->DisplayList, list);
 
    fprintf( f, "START-LIST %d, address %p\n", list, (void*)n );
 
-   done = GL_FALSE;
+   done = n ? GL_FALSE : GL_TRUE;
    while (!done) {
       opcode = n[0].opcode;
 
