@@ -1,8 +1,8 @@
-/* $Id: fakeglx.c,v 1.20 1997/08/13 01:52:18 brianp Exp $ */
+/* $Id: fakeglx.c,v 1.23 1997/10/18 20:10:27 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,15 @@
 
 /*
  * $Log: fakeglx.c,v $
+ * Revision 1.23  1997/10/18 20:10:27  brianp
+ * fixed a comment in Fake_glXMakeCurrent()
+ *
+ * Revision 1.22  1997/09/30 23:30:58  brianp
+ * modified save_glx_visual() to prevent duplicate table entries
+ *
+ * Revision 1.21  1997/09/29 23:24:40  brianp
+ * changed version to 2.5
+ *
  * Revision 1.20  1997/08/13 01:52:18  brianp
  * don't look for visuals deeper than 24 bits
  * fixed problem in glXSwapBuffers (Stephane Rehel)
@@ -269,11 +278,7 @@ save_glx_visual( Display *dpy, XVisualInfo *vinfo,
 {
    GLboolean ximage_flag = GL_TRUE;
    XMesaVisual xmvis;
-
-   if (NumVisuals>=MAX_VISUALS) {
-      fprintf( stderr, "GLX Error: maximum number of visuals exceeded\n");
-      return NULL;
-   }
+   GLint i;
 
    if (dbl) {
       /* Check if the MESA_BACK_BUFFER env var is set */
@@ -292,6 +297,30 @@ save_glx_visual( Display *dpy, XVisualInfo *vinfo,
       }
    }
 
+   /* First check if a matching visual is already in the list */
+   for (i=0; i<NumVisuals; i++) {
+      XMesaVisual v = VisualTable[i];
+      if (v->display == dpy
+          && v->vishandle->visualid == vinfo->visualid
+          && v->level == level
+          && v->ximage_flag == ximage_flag
+          && v->gl_visual->RGBAflag == rgb
+          && v->gl_visual->DBflag == dbl
+          && (v->gl_visual->DepthBits > depth_size || depth_size == 0)
+          && (v->gl_visual->StencilBits > stencil_size || stencil_size == 0)
+          && (v->gl_visual->AccumBits > accum_size || accum_size == 0)) {
+         /* this visual already in the list */
+         return v;
+      }
+   }
+
+   /* Create a new visual and add it to the list. */
+
+   if (NumVisuals>=MAX_VISUALS) {
+      fprintf( stderr, "GLX Error: maximum number of visuals exceeded\n");
+      return NULL;
+   }
+
    xmvis = XMesaCreateVisual( dpy, vinfo, rgb, alpha, dbl, ximage_flag,
                               depth_size, stencil_size, accum_size, level );
    if (xmvis) {
@@ -1006,8 +1035,7 @@ Bool Fake_glXMakeCurrent( Display *dpy, GLXDrawable drawable, GLXContext ctx )
          /* drawable must be a new window! */
          buffer = XMesaCreateWindowBuffer( ctx->xm_visual, drawable );
          if (!buffer) {
-            /* Out of memory, hit MAX_BUFFERS limit, or bad visual/drawable */
-            /* combination! */
+            /* Out of memory, or context/drawable depth mismatch */
             return False;
          }
       }
@@ -1346,7 +1374,7 @@ const char *Fake_glXQueryServerString( Display *dpy, int screen, int name )
 {
    static char *extensions = EXTENSIONS;
    static char *vendor = "Brian Paul";
-   static char *version = "1.1 Mesa 2.4";
+   static char *version = "1.1 Mesa 2.5";
 
    switch (name) {
       case GLX_EXTENSIONS:
@@ -1367,7 +1395,7 @@ const char *Fake_glXGetClientString( Display *dpy, int name )
 {
    static char *extensions = EXTENSIONS;
    static char *vendor = "Brian Paul";
-   static char *version = "1.1 Mesa 2.4";
+   static char *version = "1.1 Mesa 2.5";
 
    switch (name) {
       case GLX_EXTENSIONS:
