@@ -1,4 +1,4 @@
-/* $Id: feedback.c,v 1.10 1997/07/24 01:25:01 brianp Exp $ */
+/* $Id: feedback.c,v 1.11 1997/11/07 03:38:07 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -23,6 +23,9 @@
 
 /*
  * $Log: feedback.c,v $
+ * Revision 1.11  1997/11/07 03:38:07  brianp
+ * added stdio.h include for SunOS 4.x
+ *
  * Revision 1.10  1997/07/24 01:25:01  brianp
  * changed precompiled header symbol from PCH to PC_HEADER
  *
@@ -60,6 +63,7 @@
 #include "all.h"
 #else
 #include <assert.h>
+#include <stdio.h>
 #include "context.h"
 #include "feedback.h"
 #include "dlist.h"
