@@ -1,8 +1,8 @@
-/* $Id: misc.c,v 1.17 1997/08/13 01:26:40 brianp Exp $ */
+/* $Id: misc.c,v 1.21 1997/10/29 01:29:09 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,18 @@
 
 /*
  * $Log: misc.c,v $
+ * Revision 1.21  1997/10/29 01:29:09  brianp
+ * added GL_EXT_point_parameters extension from Daniel Barrero
+ *
+ * Revision 1.20  1997/10/16 02:32:19  brianp
+ * added GL_EXT_shared_texture_palette extension
+ *
+ * Revision 1.19  1997/10/14 00:17:48  brianp
+ * added 3DFX_set_global_palette to extension string for 3Dfx
+ *
+ * Revision 1.18  1997/09/27 00:13:44  brianp
+ * added GL_EXT_paletted_texture extension
+ *
  * Revision 1.17  1997/08/13 01:26:40  brianp
  * changed version string to 2.4
  *
@@ -273,8 +285,12 @@ const GLubyte *gl_GetString( GLcontext *ctx, GLenum name )
 {
    static char renderer[1000];
    static char *vendor = "Brian Paul";
-   static char *version = "1.1 Mesa 2.4";
-   static char *extensions = "GL_EXT_blend_color GL_EXT_blend_minmax GL_EXT_blend_logic_op GL_EXT_blend_subtract GL_EXT_polygon_offset GL_EXT_vertex_array GL_EXT_texture_object GL_EXT_texture3D GL_MESA_window_pos GL_MESA_resize_buffers";
+   static char *version = "1.1 Mesa 2.5";
+#ifdef FX
+   static char *extensions = "GL_EXT_blend_color GL_EXT_blend_minmax GL_EXT_blend_logic_op GL_EXT_blend_subtract GL_EXT_paletted_texture GL_EXT_point_parameters GL_EXT_polygon_offset GL_EXT_vertex_array GL_EXT_texture_object GL_EXT_texture3D GL_MESA_window_pos GL_MESA_resize_buffers GL_EXT_shared_texture_palette 3DFX_set_global_palette";
+#else
+   static char *extensions = "GL_EXT_blend_color GL_EXT_blend_minmax GL_EXT_blend_logic_op GL_EXT_blend_subtract GL_EXT_paletted_texture GL_EXT_point_parameters GL_EXT_polygon_offset GL_EXT_vertex_array GL_EXT_texture_object GL_EXT_texture3D GL_MESA_window_pos GL_MESA_resize_buffers GL_EXT_shared_texture_palette";
+#endif
 
    if (INSIDE_BEGIN_END(ctx)) {
       gl_error( ctx, GL_INVALID_OPERATION, "glGetString" );
