@@ -1,8 +1,8 @@
-/* $Id: mmath.h,v 1.2 1997/05/03 00:54:31 brianp Exp $ */
+/* $Id: mmath.h,v 1.4 1997/10/15 00:36:17 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.3
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,12 @@
 
 /*
  * $Log: mmath.h,v $
+ * Revision 1.4  1997/10/15 00:36:17  brianp
+ * renamed the FAST/REGULAR_MATH macros
+ *
+ * Revision 1.3  1997/09/29 22:23:54  brianp
+ * added FAST/REGULAR_MATH macros
+ *
  * Revision 1.2  1997/05/03 00:54:31  brianp
  * fixed a comment
  *
@@ -46,6 +52,23 @@
 #include <math.h>
 
 
+/*
+ * Set the x86 FPU control word to less precision for more speed:
+ */
+#if defined(__linux__) && defined(__i386__) && defined(FAST_MATH)
+#include <i386/fpu_control.h>
+#define START_FAST_MATH  __setfpucw(_FPU_SINGLE | _FPU_MASK_IM | _FPU_MASK_DM \
+            | _FPU_MASK_ZM | _FPU_MASK_OM | _FPU_MASK_UM | _FPU_MASK_PM);
+#define END_FAST_MATH  __setfpucw(_FPU_EXTENDED | _FPU_MASK_IM | _FPU_MASK_DM \
+            | _FPU_MASK_ZM | _FPU_MASK_OM | _FPU_MASK_UM | _FPU_MASK_PM);
+#else
+#define START_FAST_MATH
+#define END_FAST_MATH
+#endif
+
+
+
+
 extern float gl_sqrt(float x);
 
 
