@@ -1,8 +1,8 @@
-/* $Id: shade.c,v 1.6 1997/06/20 04:15:43 brianp Exp $ */
+/* $Id: shade.c,v 1.9 1997/07/24 01:21:56 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.3
+ * Version:  2.4
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,15 @@
 
 /*
  * $Log: shade.c,v $
+ * Revision 1.9  1997/07/24 01:21:56  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.8  1997/07/09 03:04:44  brianp
+ * fixed bug in gl_color_shade_vertices() with GL_COLOR_MATERIAL
+ *
+ * Revision 1.7  1997/07/05 16:24:26  brianp
+ * fixed FP underflow problem in pow().  Renamed h[xyz] to h_[xyz].
+ *
  * Revision 1.6  1997/06/20 04:15:43  brianp
  * optimized changing of SHININESS (Henk Kok)
  *
@@ -44,7 +53,7 @@
  */
 
 
-#ifdef PCH
+#ifdef PC_HEADER
 #include "all.h"
 #else
 #include <math.h>
@@ -56,6 +65,20 @@
 
 
 
+/*
+ * Return x^y.
+ */
+static GLfloat gl_pow( GLfloat x, GLfloat y )
+{
+   GLdouble z = pow(x, y);
+   if (z<1.0e-10)
+      return 0.0F;
+   else
+      return (GLfloat) z;
+}
+
+
+
 /*
  * Use current lighting/material settings to compute the RGBA colors of
  * an array of vertexes.
@@ -75,7 +98,7 @@ void gl_color_shade_vertices( GLcontext *ctx,
    GLint j;
    GLfloat rscale, gscale, bscale, ascale;
    GLfloat baseR, baseG, baseB, baseA;
-   GLfixed sumA;
+   GLint sumA;
    struct gl_light *light;
    struct gl_material *mat;
 
@@ -90,12 +113,12 @@ void gl_color_shade_vertices( GLcontext *ctx,
    mat = &ctx->Light.Material[side];
 
    /*** Compute color contribution from global lighting ***/
-   baseR = ctx->Light.BaseColor[side][0];
-   baseG = ctx->Light.BaseColor[side][1];
-   baseB = ctx->Light.BaseColor[side][2];
+   baseR = mat->Emission[0] + ctx->Light.Model.Ambient[0] * mat->Ambient[0];
+   baseG = mat->Emission[1] + ctx->Light.Model.Ambient[1] * mat->Ambient[1];
+   baseB = mat->Emission[2] + ctx->Light.Model.Ambient[2] * mat->Ambient[2];
    baseA = mat->Diffuse[3];  /* Alpha is simple, same for all vertices */
 
-   sumA = (GLfixed) (CLAMP( baseA, 0.0F, 1.0F ) * ascale);
+   sumA = (GLint) (CLAMP( baseA, 0.0F, 1.0F ) * ascale);
 
    for (j=0;j<n;j++) {
       GLfloat sumR, sumG, sumB;
@@ -195,7 +218,7 @@ void gl_color_shade_vertices( GLcontext *ctx,
          else {
             GLfloat diffuseR, diffuseG, diffuseB;
             GLfloat specularR, specularG, specularB;
-            GLfloat hx, hy, hz, n_dot_h, t;
+            GLfloat h_x, h_y, h_z, n_dot_h, t;
                   
             /* diffuse term */
             diffuseR = n_dot_VP * mat->Diffuse[0] * light->Diffuse[0];
@@ -216,19 +239,19 @@ void gl_color_shade_vertices( GLcontext *ctx,
                   vz *= invlen;
                }
                /* h = VP + VPe */
-               hx = VPx - vx;
-               hy = VPy - vy;
-               hz = VPz - vz;
+               h_x = VPx - vx;
+               h_y = VPy - vy;
+               h_z = VPz - vz;
             }
             else {
                /* h = VP + <0,0,1> */
-               hx = VPx;
-               hy = VPy;
-               hz = VPz + 1.0F;
+               h_x = VPx;
+               h_y = VPy;
+               h_z = VPz + 1.0F;
             }
 
             /* attention: h is not normalized, done later if needed */
-            n_dot_h = nx*hx + ny*hy + nz*hz;
+            n_dot_h = nx*h_x + ny*h_y + nz*h_z;
 
             if (n_dot_h<=0.0F) {
                specularR = 0.0F;
@@ -238,7 +261,7 @@ void gl_color_shade_vertices( GLcontext *ctx,
             else {
                GLfloat spec_coef;
                /* now `correct' the dot product */
-               n_dot_h = n_dot_h / GL_SQRT( hx*hx + hy*hy + hz*hz );
+               n_dot_h = n_dot_h / GL_SQRT( h_x*h_x + h_y*h_y + h_z*h_z );
                if (n_dot_h>1.0F) {
                   /* only happens if normal vector length > 1.0 */
                   spec_coef = pow( n_dot_h, mat->Shininess );
@@ -246,11 +269,8 @@ void gl_color_shade_vertices( GLcontext *ctx,
                else {
                   /* use table lookup approximation */
                   int k = (int) (n_dot_h * (GLfloat) (SHINE_TABLE_SIZE-1));
-#if 1
                   if (mat->ShineTable[k] < 0.0F)
-                     mat->ShineTable[k] = pow( k/(double)(SHINE_TABLE_SIZE-1),
-                                               mat->Shininess );
-#endif
+                     mat->ShineTable[k] = gl_pow( n_dot_h, mat->Shininess );
                   spec_coef = mat->ShineTable[k];
                }
                if (spec_coef<1.0e-10) {
@@ -274,9 +294,9 @@ void gl_color_shade_vertices( GLcontext *ctx,
       } /*loop over lights*/
 
       /* clamp and convert to integer or fixed point */
-      color[j][0] = (GLfixed) (CLAMP( sumR, 0.0F, 1.0F ) * rscale);
-      color[j][1] = (GLfixed) (CLAMP( sumG, 0.0F, 1.0F ) * gscale);
-      color[j][2] = (GLfixed) (CLAMP( sumB, 0.0F, 1.0F ) * bscale);
+      color[j][0] = (GLint) (CLAMP( sumR, 0.0F, 1.0F ) * rscale);
+      color[j][1] = (GLint) (CLAMP( sumG, 0.0F, 1.0F ) * gscale);
+      color[j][2] = (GLint) (CLAMP( sumB, 0.0F, 1.0F ) * bscale);
       color[j][3] = sumA;
 
    } /*loop over vertices*/
@@ -295,7 +315,7 @@ void gl_color_shade_vertices_fast( GLcontext *ctx,
 {
    GLint j;
    GLfloat rscale, gscale, bscale, ascale;
-   GLfixed sumA;
+   GLint sumA;
    GLfloat *baseColor = ctx->Light.BaseColor[side];
 
    /* Compute scale factor to go from floats in [0,1] to integers or fixed
@@ -307,7 +327,7 @@ void gl_color_shade_vertices_fast( GLcontext *ctx,
    ascale = ctx->Visual->AlphaScale;
 
    /* Alpha is easy to compute, same for all vertices */
-   sumA = (GLfixed) (baseColor[3] * ascale);
+   sumA = (GLint) (baseColor[3] * ascale);
 
    /* Loop over vertices */
    for (j=0;j<n;j++) {
@@ -379,16 +399,11 @@ void gl_color_shade_vertices_fast( GLcontext *ctx,
                else {
                   /* use table lookup approximation */
                   int k = (int) (n_dot_h * (GLfloat) (SHINE_TABLE_SIZE-1));
-#if 0
-                  GLfloat spec_coef = ctx->Light.Material[side].ShineTable[k];
-#else
                   struct gl_material *m = &ctx->Light.Material[side];
                   GLfloat spec_coef;
                   if (m->ShineTable[k] < 0.0F)
-                     m->ShineTable[k] = pow( k/(double)(SHINE_TABLE_SIZE-1),
-                                             m->Shininess );
+                     m->ShineTable[k] = gl_pow( n_dot_h, m->Shininess );
                   spec_coef = m->ShineTable[k];
-#endif
                   sumR += spec_coef * light->MatSpecular[side][0];
                   sumG += spec_coef * light->MatSpecular[side][1];
                   sumB += spec_coef * light->MatSpecular[side][2];
@@ -399,9 +414,9 @@ void gl_color_shade_vertices_fast( GLcontext *ctx,
       } /*loop over lights*/
 
       /* clamp and convert to integer or fixed point */
-      color[j][0] = (GLfixed) (MIN2( sumR, 1.0F ) * rscale);
-      color[j][1] = (GLfixed) (MIN2( sumG, 1.0F ) * gscale);
-      color[j][2] = (GLfixed) (MIN2( sumB, 1.0F ) * bscale);
+      color[j][0] = (GLint) (MIN2( sumR, 1.0F ) * rscale);
+      color[j][1] = (GLint) (MIN2( sumG, 1.0F ) * gscale);
+      color[j][2] = (GLint) (MIN2( sumB, 1.0F ) * bscale);
       color[j][3] = sumA;
 
    } /*loop over vertices*/
@@ -517,7 +532,7 @@ void gl_index_shade_vertices( GLcontext *ctx,
 
             /* accumulate specular term */
             {
-               GLfloat hx, hy, hz, dot, spec_coef;
+               GLfloat h_x, h_y, h_z, n_dot_h, spec_coef;
 
                /* specular term */
                if (ctx->Light.Model.LocalViewer) {
@@ -532,29 +547,31 @@ void gl_index_shade_vertices( GLcontext *ctx,
                      vy *= invlen;
                      vz *= invlen;
                   }
-                  hx = lx - vx;
-                  hy = ly - vy;
-                  hz = lz - vz;
+                  h_x = lx - vx;
+                  h_y = ly - vy;
+                  h_z = lz - vz;
                }
                else {
-                  hx = lx;
-                  hy = ly;
-                  hz = lz + 1.0F;
+                  h_x = lx;
+                  h_y = ly;
+                  h_z = lz + 1.0F;
                }
                /* attention: s is not normalized, done later if necessary */
-               dot = hx*nx + hy*ny + hz*nz;
+               n_dot_h = h_x*nx + h_y*ny + h_z*nz;
 
-               if (dot<=0.0F) {
+               if (n_dot_h <= 0.0F) {
                   spec_coef = 0.0F;
                }
                else {
                   /* now `correct' the dot product */
-                  dot = dot / GL_SQRT(hx*hx + hy*hy + hz*hz);
-                  if (dot>1.0F) {
-                     spec_coef = pow( dot, mat->Shininess );
+                  n_dot_h = n_dot_h / GL_SQRT(h_x*h_x + h_y*h_y + h_z*h_z);
+                  if (n_dot_h>1.0F) {
+                     spec_coef = pow( n_dot_h, mat->Shininess );
                   }
                   else {
-                     int k = (int) (dot * (GLfloat)(SHINE_TABLE_SIZE-1));
+                     int k = (int) (n_dot_h * (GLfloat)(SHINE_TABLE_SIZE-1));
+                     if (mat->ShineTable[k] < 0.0F)
+                        mat->ShineTable[k] = gl_pow( n_dot_h, mat->Shininess );
                      spec_coef = mat->ShineTable[k];
                   }
                }
