@@ -1,8 +1,8 @@
-/* $Id: teximage.c,v 1.24 1997/09/03 13:17:17 brianp Exp $ */
+/* $Id: teximage.c,v 1.31 1997/11/07 03:38:07 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,27 @@
 
 /*
  * $Log: teximage.c,v $
+ * Revision 1.31  1997/11/07 03:38:07  brianp
+ * added stdio.h include for SunOS 4.x
+ *
+ * Revision 1.30  1997/11/02 20:20:30  brianp
+ * rewrote gl_TexSubImage[123]D()
+ *
+ * Revision 1.29  1997/10/16 01:14:04  brianp
+ * removed teximage Dirty flag
+ *
+ * Revision 1.28  1997/10/14 00:40:20  brianp
+ * added DavidB's v19 fxmesa changes
+ *
+ * Revision 1.27  1997/09/29 23:28:14  brianp
+ * updated for new device driver texture functions
+ *
+ * Revision 1.26  1997/09/28 15:29:03  brianp
+ * initialize image->Depth = 1 in read_color_image() (Hiroki Honda)
+ *
+ * Revision 1.25  1997/09/27 00:14:39  brianp
+ * added GL_EXT_paletted_texture extension
+ *
  * Revision 1.24  1997/09/03 13:17:17  brianp
  * added a few pointer casts
  *
@@ -102,6 +123,7 @@
 #include "all.h"
 #else
 #include <assert.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "context.h"
@@ -209,6 +231,13 @@ static GLint decode_internal_format( GLint format )
       case GL_RGBA12:
       case GL_RGBA16:
          return GL_RGBA;
+      case GL_COLOR_INDEX1_EXT:
+      case GL_COLOR_INDEX2_EXT:
+      case GL_COLOR_INDEX4_EXT:
+      case GL_COLOR_INDEX8_EXT:
+      case GL_COLOR_INDEX12_EXT:
+      case GL_COLOR_INDEX16_EXT:
+         return GL_COLOR_INDEX;
       default:
          return -1;  /* error */
    }
@@ -273,6 +302,13 @@ static GLint components_in_intformat( GLint format )
       case GL_RGBA12:
       case GL_RGBA16:
          return 4;
+      case GL_COLOR_INDEX1_EXT:
+      case GL_COLOR_INDEX2_EXT:
+      case GL_COLOR_INDEX4_EXT:
+      case GL_COLOR_INDEX8_EXT:
+      case GL_COLOR_INDEX12_EXT:
+      case GL_COLOR_INDEX16_EXT:
+         return 1;
       default:
          return -1;  /* error */
    }
@@ -299,15 +335,15 @@ void gl_free_texture_image( struct gl_texture_image *teximage )
 
 /*
  * Given a gl_image, apply the pixel transfer scale, bias, and mapping
- * to produce a gl_texture_image.
+ * to produce a gl_texture_image.  Convert image data to GLubytes.
  * Input:  image - the incoming gl_image
  *         internalFormat - desired format of resultant texture
  *         border - texture border width (0 or 1)
  * Return:  pointer to a gl_texture_image or NULL if an error occurs.
  */
 static struct gl_texture_image *
-gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
-                     GLenum internalFormat, GLint border )
+image_to_texture( GLcontext *ctx, const struct gl_image *image,
+                  GLenum internalFormat, GLint border )
 {
    GLint components;
    struct gl_texture_image *texImage;
@@ -319,7 +355,7 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
    assert(image->Height>0);
    assert(image->Depth>0);
 
-   internalFormat = decode_internal_format(internalFormat);
+   /*   internalFormat = decode_internal_format(internalFormat);*/
    components = components_in_intformat(internalFormat);
    numPixels = image->Width * image->Height * image->Depth;
 
@@ -327,7 +363,8 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
    if (!texImage)
       return NULL;
 
-   texImage->Format = internalFormat;
+   texImage->Format = decode_internal_format(internalFormat);
+   texImage->IntFormat = internalFormat;
    texImage->Border = border;
    texImage->Width = image->Width;
    texImage->Height = image->Height;
@@ -411,7 +448,7 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
 
                   /* store texel (components are GLubytes in [0,255]) */
                   pixel = j * image->Width + i;
-                  switch (internalFormat) {
+                  switch (texImage->Format) {
                      case GL_ALPHA:
                         texImage->Data[pixel] = alpha;
                         break;
@@ -437,7 +474,7 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
                         texImage->Data[pixel*4+3] = alpha;
                         break;
                      default:
-                        gl_problem(ctx,"Bad format in gl_image_to_texture");
+                        gl_problem(ctx,"Bad format in image_to_texture");
                         return NULL;
                   }
                }
@@ -453,12 +490,18 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
             GLubyte red, green, blue, alpha;
             switch (image->Format) {
                case GL_COLOR_INDEX:
-                  {
+                  if (decode_internal_format(internalFormat)==GL_COLOR_INDEX) {
+                     /* a paletted texture */
+                     GLint index = ((GLubyte*)image->Data)[pixel];
+                     red = index;
+                  }
+                  else {
+                     /* convert color index to RGBA */
                      GLint index = ((GLubyte*)image->Data)[pixel];
                      red   = 255.0F * ctx->Pixel.MapItoR[index];
                      green = 255.0F * ctx->Pixel.MapItoG[index];
                      blue  = 255.0F * ctx->Pixel.MapItoB[index];
-                     alpha = 255;
+                     alpha = 255.0F * ctx->Pixel.MapItoA[index];
                   }
                   break;
                case GL_RGB:
@@ -511,7 +554,7 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
                   alpha = ((GLubyte*) image->Data)[pixel*2+1];
                   break;
               default:
-                 gl_problem(ctx,"Bad format (2) in gl_image_to_texture");
+                 gl_problem(ctx,"Bad format (2) in image_to_texture");
                  return NULL;
             }
             
@@ -550,7 +593,10 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
             }
 
             /* store texel (components are GLubytes in [0,255]) */
-            switch (internalFormat) {
+            switch (texImage->Format) {
+               case GL_COLOR_INDEX:
+                  texImage->Data[pixel] = red; /* really an index */
+                  break;
                case GL_ALPHA:
                   texImage->Data[pixel] = alpha;
                   break;
@@ -576,7 +622,7 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
                   texImage->Data[pixel*4+3] = alpha;
                   break;
                default:
-                  gl_problem(ctx,"Bad format (3) in gl_image_to_texture");
+                  gl_problem(ctx,"Bad format (3) in image_to_texture");
                   return NULL;
             }
          }
@@ -585,9 +631,14 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
       case GL_FLOAT:
          for (pixel=0; pixel<numPixels; pixel++) {
             GLfloat red, green, blue, alpha;
-            switch (image->Format) {
+            switch (texImage->Format) {
                case GL_COLOR_INDEX:
-                  {
+                  if (decode_internal_format(internalFormat)==GL_COLOR_INDEX) {
+                     /* a paletted texture */
+                     GLint index = (GLint) ((GLfloat*) image->Data)[pixel];
+                     red = index;
+                  }
+                  else {
                      GLint shift = ctx->Pixel.IndexShift;
                      GLint offset = ctx->Pixel.IndexOffset;
                      /* MapIto[RGBA]Size must be powers of two */
@@ -661,7 +712,7 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
                   alpha = ((GLfloat*) image->Data)[pixel*2+1];
                   break;
                default:
-                  gl_problem(ctx,"Bad format (4) in gl_image_to_texture");
+                  gl_problem(ctx,"Bad format (4) in image_to_texture");
                   return NULL;
             }
             
@@ -691,7 +742,11 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
             }
 
             /* store texel (components are GLubytes in [0,255]) */
-            switch (internalFormat) {
+            switch (texImage->Format) {
+               case GL_COLOR_INDEX:
+                  /* a paletted texture */
+                  texImage->Data[pixel] = (GLint) (red * 255.0F);
+                  break;
                case GL_ALPHA:
                   texImage->Data[pixel] = (GLint) (alpha * 255.0F);
                   break;
@@ -717,14 +772,14 @@ gl_image_to_texture( GLcontext *ctx, const struct gl_image *image,
                   texImage->Data[pixel*4+3] = (GLint) (alpha * 255.0F);
                   break;
                default:
-                  gl_problem(ctx,"Bad format (5) in gl_image_to_texture");
+                  gl_problem(ctx,"Bad format (5) in image_to_texture");
                   return NULL;
             }
          }
          break;
 
       default:
-         gl_problem(ctx, "Bad image type in gl_image_to_texture");
+         gl_problem(ctx, "Bad image type in image_to_texture");
          return NULL;
    }
 
@@ -746,7 +801,7 @@ make_null_texture( GLcontext *ctx, GLenum internalFormat,
    struct gl_texture_image *texImage;
    GLint numPixels;
 
-   internalFormat = decode_internal_format(internalFormat);
+   /*internalFormat = decode_internal_format(internalFormat);*/
    components = components_in_intformat(internalFormat);
    numPixels = width * height * depth;
 
@@ -754,7 +809,8 @@ make_null_texture( GLcontext *ctx, GLenum internalFormat,
    if (!texImage)
       return NULL;
 
-   texImage->Format = internalFormat;
+   texImage->Format = decode_internal_format(internalFormat);
+   texImage->IntFormat = internalFormat;
    texImage->Border = border;
    texImage->Width = width;
    texImage->Height = height;
@@ -1099,7 +1155,7 @@ void gl_TexImage1D( GLcontext *ctx,
 
       /* make new texture from source image */
       if (image) {
-         teximage = gl_image_to_texture(ctx, image, internalformat, border);
+         teximage = image_to_texture(ctx, image, internalformat, border);
       }
       else {
          teximage = make_null_texture(ctx, internalformat,
@@ -1117,10 +1173,10 @@ void gl_TexImage1D( GLcontext *ctx,
          gl_free_image(image);
       }
 
-      /* Send texture image to device driver */
+      /* tell driver about change */
       if (ctx->Driver.TexImage) {
          (*ctx->Driver.TexImage)( ctx, GL_TEXTURE_1D,
-                                  ctx->Texture.Current1D->Name,
+                                  ctx->Texture.Current1D,
                                   level, internalformat, teximage );
       }
    }
@@ -1183,7 +1239,7 @@ void gl_TexImage2D( GLcontext *ctx,
 
       /* make new texture from source image */
       if (image) {
-         teximage = gl_image_to_texture(ctx, image, internalformat, border);
+         teximage = image_to_texture(ctx, image, internalformat, border);
       }
       else {
          teximage = make_null_texture(ctx, internalformat,
@@ -1201,10 +1257,10 @@ void gl_TexImage2D( GLcontext *ctx,
          gl_free_image(image);
       }
 
-      /* Send texture image to device driver */
+      /* tell driver about change */
       if (ctx->Driver.TexImage) {
          (*ctx->Driver.TexImage)( ctx, GL_TEXTURE_2D,
-                                  ctx->Texture.Current2D->Name,
+                                  ctx->Texture.Current2D,
                                   level, internalformat, teximage );
       }
    }
@@ -1267,7 +1323,7 @@ void gl_TexImage3DEXT( GLcontext *ctx,
 
       /* make new texture from source image */
       if (image) {
-         teximage = gl_image_to_texture(ctx, image, internalformat, border);
+         teximage = image_to_texture(ctx, image, internalformat, border);
       }
       else {
          teximage = make_null_texture(ctx, internalformat,
@@ -1285,8 +1341,12 @@ void gl_TexImage3DEXT( GLcontext *ctx,
          gl_free_image(image);
       }
 
-      /* Send texture image to device driver */
-      /* TODO: device driver 3-D texture function */
+      /* tell driver about change */
+      if (ctx->Driver.TexImage) {
+         (*ctx->Driver.TexImage)( ctx, GL_TEXTURE_3D_EXT,
+                                  ctx->Texture.Current3D,
+                                  level, internalformat, teximage );
+      }
    }
    else if (target==GL_PROXY_TEXTURE_3D_EXT) {
       /* Proxy texture: check for errors and update proxy state */
@@ -1317,42 +1377,31 @@ void gl_TexImage3DEXT( GLcontext *ctx,
 }
 
 
+
 void gl_GetTexImage( GLcontext *ctx, GLenum target, GLint level, GLenum format,
                      GLenum type, GLvoid *pixels )
 {
+   gl_problem(ctx, "glGetTexImage not implemented");
    /* TODO */
 }
 
 
 
 
-
 /*
- * GL_EXT_copy_texture
- */
-
-
-
-/*
- * Unpack the image data given to glTexSubImage[12]D.
+ * Unpack the image data given to glTexSubImage[123]D.
+ * This function is just a wrapper for gl_unpack_image() but it does
+ * some extra error checking.
  */
 struct gl_image *
-gl_unpack_texsubimage( GLcontext *ctx, GLint width, GLint height,
+gl_unpack_texsubimage( GLcontext *ctx, GLint width, GLint height, GLint depth,
                        GLenum format, GLenum type, const GLvoid *pixels )
 {
-   GLint components;
-   GLenum desttype;
-
-   if (width<0 || height<0 || !pixels) {
-      return NULL;
-   }
-
    if (type==GL_BITMAP && format!=GL_COLOR_INDEX) {
       return NULL;
    }
 
-   components = components_in_intformat( format );
-   if (components<0 || format==GL_STENCIL_INDEX || format==GL_DEPTH_COMPONENT){
+   if (format==GL_STENCIL_INDEX || format==GL_DEPTH_COMPONENT){
       return NULL;
    }
 
@@ -1360,55 +1409,7 @@ gl_unpack_texsubimage( GLcontext *ctx, GLint width, GLint height,
       return NULL;
    }
 
-   if (type==GL_UNSIGNED_BYTE) {
-      desttype = GL_UNSIGNED_BYTE;
-   }
-   else {
-      desttype = GL_FLOAT;
-   }
-   
-   return gl_unpack_image( ctx, width, height, format, type,
-                           desttype, pixels, GL_FALSE );
-}
-
-
-/*
- * Unpack the image data given to glTexSubImage3D.
- */
-struct gl_image *
-gl_unpack_texsubimage3D( GLcontext *ctx, GLint width, GLint height,
-                         GLint depth, GLenum format, GLenum type,
-                         const GLvoid *pixels )
-{
-   GLint components;
-   GLenum desttype;
-
-   if (width<0 || height<0 || depth<0 || !pixels) {
-      return NULL;
-   }
-
-   if (type==GL_BITMAP && format!=GL_COLOR_INDEX) {
-      return NULL;
-   }
-
-   components = components_in_intformat( format );
-   if (components<0 || format==GL_STENCIL_INDEX || format==GL_DEPTH_COMPONENT) {
-      return NULL;
-   }
-
-   if (gl_sizeof_type(type)<=0) {
-      return NULL;
-   }
-
-   if (type==GL_UNSIGNED_BYTE) {
-      desttype = GL_UNSIGNED_BYTE;
-   }
-   else {
-      desttype = GL_FLOAT;
-   }
-  
-   return gl_unpack_image3D( ctx, width, height, depth, format, type,
-                           desttype, pixels, GL_FALSE );
+   return gl_unpack_image3D( ctx, width, height, depth, format, type, pixels );
 }
 
 
@@ -1418,7 +1419,7 @@ void gl_TexSubImage1D( GLcontext *ctx,
                        GLsizei width, GLenum format, GLenum type,
                        struct gl_image *image )
 {
-   struct gl_texture_image *teximage;
+   struct gl_texture_image *destTex;
 
    if (target!=GL_TEXTURE_1D) {
       gl_error( ctx, GL_INVALID_ENUM, "glTexSubImage1D(target)" );
@@ -1429,56 +1430,56 @@ void gl_TexSubImage1D( GLcontext *ctx,
       return;
    }
 
-   teximage = ctx->Texture.Current1D->Image[level];
-   if (!teximage) {
+   destTex = ctx->Texture.Current1D->Image[level];
+   if (!destTex) {
       gl_error( ctx, GL_INVALID_OPERATION, "glTexSubImage1D" );
       return;
    }
 
-   if (xoffset < -teximage->Border) {
+   if (xoffset < -destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage1D(xoffset)" );
       return;
    }
-   if (xoffset + width > teximage->Width+teximage->Border) {
+   if (xoffset + width > destTex->Width + destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage1D(xoffset+width)" );
       return;
    }
 
    if (image) {
       /* unpacking must have been error-free */
-      GLint texcomponents, i, k;
+      GLint texcomponents = components_in_intformat(destTex->Format);
 
-      /* TODO: this is temporary.  If Type==GL_FLOAT or scale&bias needed */
-      /* then do more work. */
-      if (image->Type==GL_FLOAT) {
-         gl_problem( ctx, "unimplemented texture type in glTexSubImage1D" );
-         return;
-      }
-
-      texcomponents = components_in_intformat(teximage->Format);
-
-      if (texcomponents == image->Components) {
+      if (image->Type==GL_UNSIGNED_BYTE && texcomponents==image->Components) {
+         /* Simple case, just byte copy image data into texture image */
+         /* row by row. */
+         GLubyte *dst = destTex->Data + texcomponents * xoffset;
          GLubyte *src = (GLubyte *) image->Data;
-         GLubyte *dst = teximage->Data + texcomponents * xoffset;
          MEMCPY( dst, src, width * texcomponents );
       }
       else {
-         /* TODO: this is a hack */
-         GLubyte *src = (GLubyte *) image->Data;
-         GLubyte *dst = teximage->Data + texcomponents * xoffset;
-         gl_problem( ctx, "component mismatch in glTexSubImage1D" );
-         for (i=0;i<width;i++) {
-            for (k=0;k<texcomponents;k++) {
-               dst[k] = src[k];
-            }
-            dst += texcomponents;
-            src += image->Components;
-         }
+         /* General case, convert image pixels into texels, scale, bias, etc */
+         struct gl_texture_image *subTexImg = image_to_texture(ctx, image,
+                                        destTex->IntFormat, destTex->Border);
+         GLubyte *dst = destTex->Data + texcomponents * xoffset;
+         GLubyte *src = subTexImg->Data;
+         MEMCPY( dst, src, width * texcomponents );
+         gl_free_texture_image(subTexImg);
       }
+
       /* if the image's reference count is zero, delete it now */
       if (image->RefCount==0) {
          gl_free_image(image);
       }
+
+      ctx->Texture.Current1D->Dirty = GL_TRUE;
+
+      /* tell driver about change */
+      if (ctx->Driver.TexImage) {
+         (*ctx->Driver.TexImage)( ctx, GL_TEXTURE_1D,
+                                  ctx->Texture.Current1D,
+                                  level, ctx->Texture.Current1D->Image[level]->IntFormat,
+				  destTex );
+      }
    }
    else {
       /* if no image, an error must have occured, do more testing now */
@@ -1506,7 +1507,6 @@ void gl_TexSubImage1D( GLcontext *ctx,
       /* if we get here, probably ran out of memory during unpacking */
       gl_error( ctx, GL_OUT_OF_MEMORY, "glTexSubImage1D" );
    }
-   ctx->Texture.Current1D->Dirty = GL_TRUE;
 }
 
 
@@ -1518,7 +1518,7 @@ void gl_TexSubImage2D( GLcontext *ctx,
                        GLenum format, GLenum type,
                        struct gl_image *image )
 {
-   struct gl_texture_image *teximage;
+   struct gl_texture_image *destTex;
 
    if (target!=GL_TEXTURE_2D) {
       gl_error( ctx, GL_INVALID_ENUM, "glTexSubImage2D(target)" );
@@ -1529,74 +1529,75 @@ void gl_TexSubImage2D( GLcontext *ctx,
       return;
    }
 
-   teximage = ctx->Texture.Current2D->Image[level];
-   if (!teximage) {
+   destTex = ctx->Texture.Current2D->Image[level];
+   if (!destTex) {
       gl_error( ctx, GL_INVALID_OPERATION, "glTexSubImage2D" );
       return;
    }
 
-   if (xoffset < -teximage->Border) {
+   if (xoffset < -destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage2D(xoffset)" );
       return;
    }
-   if (yoffset < -teximage->Border) {
+   if (yoffset < -destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage2D(yoffset)" );
       return;
    }
-   if (xoffset + width > teximage->Width+teximage->Border) {
+   if (xoffset + width > destTex->Width + destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage2D(xoffset+width)" );
       return;
    }
-   if (yoffset + height > teximage->Height+teximage->Border) {
+   if (yoffset + height > destTex->Height + destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage2D(yoffset+height)" );
       return;
    }
 
    if (image) {
       /* unpacking must have been error-free */
-      GLint texcomponents, i, j, k;
+      GLint texcomponents = components_in_intformat(destTex->Format);
 
-      /* TODO: this is temporary.  If Type==GL_FLOAT or scale&bias needed */
-      /* then do more work. */
-      if (image->Type==GL_FLOAT) {
-         gl_problem( ctx, "unimplemented texture type in glTexSubImage2D" );
-         return;
-      }
-
-      texcomponents = components_in_intformat(teximage->Format);
-
-      if (texcomponents == image->Components) {
-         GLubyte *dst = teximage->Data 
-                      + (yoffset * teximage->Width + xoffset) * texcomponents;
+      if (image->Type==GL_UNSIGNED_BYTE && texcomponents==image->Components) {
+         /* Simple case, just byte copy image data into texture image */
+         /* row by row. */
+         GLubyte *dst = destTex->Data 
+                      + (yoffset * destTex->Width + xoffset) * texcomponents;
          GLubyte *src = (GLubyte *) image->Data;
+         GLint  j;
          for (j=0;j<height;j++) {
             MEMCPY( dst, src, width * texcomponents );
-            dst += teximage->Width * texcomponents;
-            src += width * texcomponents;
+            dst += destTex->Width * texcomponents * sizeof(GLubyte);
+            src += width * texcomponents * sizeof(GLubyte);
          }
       }
       else {
-         /* TODO: this is a hack */
-         gl_problem( ctx, "component mismatch in glTexSubImage2D" );
-
+         /* General case, convert image pixels into texels, scale, bias, etc */
+         struct gl_texture_image *subTexImg = image_to_texture(ctx, image,
+                                        destTex->IntFormat, destTex->Border);
+         GLubyte *dst = destTex->Data
+                  + (yoffset * destTex->Width + xoffset) * texcomponents;
+         GLubyte *src = subTexImg->Data;
+         GLint j;
          for (j=0;j<height;j++) {
-            GLubyte *dst = teximage->Data 
-                  + ((yoffset+j) * teximage->Width + xoffset) * texcomponents;
-            GLubyte *src = (GLubyte *) image->Data
-                         + j * width * image->Components;
-            for (i=0;i<width;i++) {
-               for (k=0;k<texcomponents;k++) {
-                  dst[k] = src[k];
-               }
-               dst += texcomponents;
-               src += image->Components;
-            }
+            MEMCPY( dst, src, width * texcomponents );
+            dst += destTex->Width * texcomponents * sizeof(GLubyte);
+            src += width * texcomponents * sizeof(GLubyte);
          }
+         gl_free_texture_image(subTexImg);
       }
+
       /* if the image's reference count is zero, delete it now */
       if (image->RefCount==0) {
          gl_free_image(image);
       }
+
+      ctx->Texture.Current2D->Dirty = GL_TRUE;
+
+      /* tell driver about change */
+      if (ctx->Driver.TexImage) {
+         (*ctx->Driver.TexImage)( ctx, GL_TEXTURE_2D, ctx->Texture.Current2D,
+                                  level, ctx->Texture.Current2D->Image[level]->IntFormat,
+				  destTex );
+      }
    }
    else {
       /* if no image, an error must have occured, do more testing now */
@@ -1614,7 +1615,7 @@ void gl_TexSubImage2D( GLcontext *ctx,
          gl_error( ctx, GL_INVALID_ENUM, "glTexSubImage1D(format)" );
          return;
       }
-      components = components_in_intformat( format );
+      components = gl_components_in_format( format );
       if (components<0 || format==GL_STENCIL_INDEX
           || format==GL_DEPTH_COMPONENT){
          gl_error( ctx, GL_INVALID_ENUM, "glTexSubImage2D(format)" );
@@ -1628,7 +1629,6 @@ void gl_TexSubImage2D( GLcontext *ctx,
       /* if we get here, probably ran out of memory during unpacking */
       gl_error( ctx, GL_OUT_OF_MEMORY, "glTexSubImage2D" );
    }
-   ctx->Texture.Current2D->Dirty = GL_TRUE;
 }
 
 
@@ -1640,7 +1640,7 @@ void gl_TexSubImage3DEXT( GLcontext *ctx,
                           GLenum format, GLenum type,
                           struct gl_image *image )
 {
-   struct gl_texture_image *teximage;
+   struct gl_texture_image *destTex;
 
    if (target!=GL_TEXTURE_3D_EXT) {
       gl_error( ctx, GL_INVALID_ENUM, "glTexSubImage3DEXT(target)" );
@@ -1651,95 +1651,95 @@ void gl_TexSubImage3DEXT( GLcontext *ctx,
       return;
    }
 
-   teximage = ctx->Texture.Current3D->Image[level];
-   if (!teximage) {
+   destTex = ctx->Texture.Current3D->Image[level];
+   if (!destTex) {
       gl_error( ctx, GL_INVALID_OPERATION, "glTexSubImage3DEXT" );
       return;
    }
 
-   if (xoffset < -teximage->Border) {
+   if (xoffset < -destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage3DEXT(xoffset)" );
       return;
    }
-   if (yoffset < -teximage->Border) {
+   if (yoffset < -destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage3DEXT(yoffset)" );
       return;
    }
-   if (zoffset < -teximage->Border) {
+   if (zoffset < -destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage3DEXT(zoffset)" );
       return;
    }
-   if (xoffset + width > teximage->Width+teximage->Border) {
+   if (xoffset + width > destTex->Width+destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage3DEXT(xoffset+width)" );
       return;
    }
-   if (yoffset + height > teximage->Height+teximage->Border) {
+   if (yoffset + height > destTex->Height+destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage3DEXT(yoffset+height)" );
       return;
    }
-   if (zoffset + depth  > teximage->Depth+teximage->Border) {
+   if (zoffset + depth  > destTex->Depth+destTex->Border) {
       gl_error( ctx, GL_INVALID_VALUE, "glTexSubImage3DEXT(zoffset+depth)" );
       return;
    }
 
    if (image) {
       /* unpacking must have been error-free */
-      GLint texcomponents, i, j, k, rectarea;
+      GLint texcomponents = components_in_intformat(destTex->Format);
+      GLint dstRectArea = destTex->Width * destTex->Height;
+      GLint srcRectArea = width * height;
 
-      /* TODO: this is temporary.  If Type==GL_FLOAT or scale&bias needed */
-      /* then do more work. */
-      if (image->Type==GL_FLOAT) {
-         gl_problem( ctx, "unimplemented texture type in glTexSubImagDEXT" );
-         return;
-      }
-
-      texcomponents = components_in_intformat(teximage->Format);
-      rectarea=teximage->Width * teximage->Height;
-
-      if (texcomponents == image->Components) {
-         GLubyte *dst = teximage->Data
-               + (zoffset * rectarea +  yoffset * teximage->Width + xoffset)
+      if (image->Type==GL_UNSIGNED_BYTE && texcomponents==image->Components) {
+         /* Simple case, just byte copy image data into texture image */
+         /* row by row. */
+         GLubyte *dst = destTex->Data 
+               + (zoffset * dstRectArea +  yoffset * destTex->Width + xoffset)
                * texcomponents;
          GLubyte *src = (GLubyte *) image->Data;
+         GLint j, k;
          for(k=0;k<depth; k++) {
            for (j=0;j<height;j++) {
               MEMCPY( dst, src, width * texcomponents );
-              dst += teximage->Width * texcomponents;
+              dst += destTex->Width * texcomponents;
               src += width * texcomponents;
            }
-           dst += rectarea * texcomponents;
-           src += rectarea * texcomponents;
+           dst += dstRectArea * texcomponents * sizeof(GLubyte);
+           src += srcRectArea * texcomponents * sizeof(GLubyte);
          }
       }
       else {
-         /* TODO: this is a hack */
-         GLint l, trarea, tzoffset, tloffset;
-         GLubyte *dst, *src;
-
-         gl_problem( ctx, "component mismatch in glTexSubImage3DEXT" );
-         trarea=width*depth * image->Components;
-         for (l=0; l<depth; l++) {
-            tzoffset=(zoffset+l) * rectarea;
-            tloffset=l * trarea;
-            for (j=0;j<height;j++) {
-               dst = teximage->Data
-                   + (tzoffset + (yoffset+j) * teximage->Width + xoffset)
-                   * texcomponents;
-               src = (GLubyte *) image->Data + tloffset + j * width * image->Components;
-               for (i=0;i<width;i++) {
-                  for (k=0;k<texcomponents;k++) {
-                     dst[k] = src[k];
-                  }
-                  dst += texcomponents;
-                  src += image->Components;
-               }
-            }
+         /* General case, convert image pixels into texels, scale, bias, etc */
+         struct gl_texture_image *subTexImg = image_to_texture(ctx, image,
+                                        destTex->IntFormat, destTex->Border);
+         GLubyte *dst = destTex->Data 
+               + (zoffset * dstRectArea +  yoffset * destTex->Width + xoffset)
+               * texcomponents;
+         GLubyte *src = subTexImg->Data;
+         GLint j, k;
+         for(k=0;k<depth; k++) {
+           for (j=0;j<height;j++) {
+              MEMCPY( dst, src, width * texcomponents );
+              dst += destTex->Width * texcomponents;
+              src += width * texcomponents;
+           }
+           dst += dstRectArea * texcomponents * sizeof(GLubyte);
+           src += srcRectArea * texcomponents * sizeof(GLubyte);
          }
+         gl_free_texture_image(subTexImg);
       }
       /* if the image's reference count is zero, delete it now */
       if (image->RefCount==0) {
          gl_free_image(image);
       }
+
+      ctx->Texture.Current3D->Dirty = GL_TRUE;
+
+      /* tell driver about change */
+      if (ctx->Driver.TexImage) {
+         (*ctx->Driver.TexImage)( ctx, GL_TEXTURE_3D_EXT,
+                                  ctx->Texture.Current3D,
+                                  level, ctx->Texture.Current3D->Image[level]->IntFormat,
+				  destTex );
+      }
    }
    else {
       /* if no image, an error must have occured, do more testing now */
@@ -1775,7 +1775,6 @@ void gl_TexSubImage3DEXT( GLcontext *ctx,
       /* if we get here, probably ran out of memory during unpacking */
       gl_error( ctx, GL_OUT_OF_MEMORY, "glTexSubImage3DEXT" );
    }
-   ctx->Texture.Current3D->Dirty = GL_TRUE;
 }
 
 
@@ -1806,10 +1805,10 @@ static struct gl_image *read_color_image( GLcontext *ctx, GLint x, GLint y,
    if (image) {
       image->Width = width;
       image->Height = height;
+      image->Depth = 1;
       image->Components = components;
       image->Format = format;
       image->Type = GL_UNSIGNED_BYTE;
-      image->Interleaved = GL_FALSE;
       image->RefCount = 0;
       image->Data = (GLubyte *) malloc( width * height * components );
       if (!image->Data) {
