@@ -1,8 +1,8 @@
-/* $Id: texobj.c,v 1.14 1997/08/23 18:41:40 brianp Exp $ */
+/* $Id: texobj.c,v 1.19 1997/11/07 03:38:07 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,21 @@
 
 /*
  * $Log: texobj.c,v $
+ * Revision 1.19  1997/11/07 03:38:07  brianp
+ * added stdio.h include for SunOS 4.x
+ *
+ * Revision 1.18  1997/10/13 23:57:59  brianp
+ * added target parameter to Driver.BindTexture()
+ *
+ * Revision 1.17  1997/09/29 23:28:14  brianp
+ * updated for new device driver texture functions
+ *
+ * Revision 1.16  1997/09/27 00:14:39  brianp
+ * added GL_EXT_paletted_texture extension
+ *
+ * Revision 1.15  1997/09/23 00:58:15  brianp
+ * now using hash table for texture objects
+ *
  * Revision 1.14  1997/08/23 18:41:40  brianp
  * fixed bug in glBindTexture() when binding an incomplete texture image
  *
@@ -72,8 +87,10 @@
 #include "all.h"
 #else
 #include <assert.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include "context.h"
+#include "hash.h"
 #include "macros.h"
 #include "teximage.h"
 #include "texobj.h"
@@ -81,84 +98,109 @@
 #endif
 
 
+
 /*
- * Allocate a new texture object structure.  The name and dimensionality are
- * set to zero here and must be initialized by the caller.
+ * Allocate a new texture object and add it to the linked list of texture
+ * objects.  If name>0 then also insert the new texture object into the hash
+ * table.
+ * Input:  shared - the shared GL state structure to contain the texture object
+ *         name - integer name for the texture object
+ *         dimensions - either 1, 2 or 3
+ * Return:  pointer to new texture object
  */
-struct gl_texture_object *gl_alloc_texture_object( void )
+struct gl_texture_object *
+gl_alloc_texture_object( struct gl_shared_state *shared, GLuint name,
+                         GLuint dimensions)
 {
    struct gl_texture_object *obj;
 
+   assert(dimensions >= 1 && dimensions <= 3);
+
    obj = (struct gl_texture_object *)
                      calloc(1,sizeof(struct gl_texture_object));
    if (obj) {
       /* init the non-zero fields */
+      obj->Name = name;
+      obj->Dimensions = dimensions;
       obj->WrapS = GL_REPEAT;
       obj->WrapT = GL_REPEAT;
       obj->MinFilter = GL_NEAREST_MIPMAP_LINEAR;
       obj->MagFilter = GL_LINEAR;
       obj->MinMagThresh = 0.0F;
+      obj->Palette[0] = 255;
+      obj->Palette[1] = 255;
+      obj->Palette[2] = 255;
+      obj->Palette[3] = 255;
+      obj->PaletteSize = 1;
+      obj->PaletteIntFormat = GL_RGBA;
+      obj->PaletteFormat = GL_RGBA;
+
+      /* insert into linked list */
+      if (shared) {
+         obj->Next = shared->TexObjectList;
+         shared->TexObjectList = obj;
+      }
+
+      if (name > 0) {
+         /* insert into hash table */
+         HashInsert(shared->TexObjects, name, obj);
+      }
    }
    return obj;
 }
 
 
 /*
- * Append a gl_texture_object struct to a list of texture objects.
+ * Deallocate a texture object struct and remove it from the given
+ * shared GL state.
+ * Input:  shared - the shared GL state to which the object belongs
+ *         t - the texture object to delete
  */
-static void append_texture_object( struct gl_texture_object *list,
-                                   struct gl_texture_object *obj )
+void gl_free_texture_object( struct gl_shared_state *shared,
+                             struct gl_texture_object *t )
 {
-   struct gl_texture_object *t;
+   struct gl_texture_object *tprev, *tcurr;
 
-   t = list;
-   assert(t);  /* always have proxy textures in the list */
-   while (t->Next) {
-      t = t->Next;
-   }
-   t->Next = obj;
-}
+   assert(t);
 
-
-
-/*
- * Deallocate a texture object struct and all children structures
- * and image data.
- */
-void gl_free_texture_object( struct gl_texture_object *t )
-{
-   GLuint i;
-
-   for (i=0;i<MAX_TEXTURE_LEVELS;i++) {
-      if (t->Image[i]) {
-         gl_free_texture_image( t->Image[i] );
+   /* unlink t from the linked list */
+   if (shared) {
+      tprev = NULL;
+      tcurr = shared->TexObjectList;
+      while (tcurr) {
+         if (tcurr==t) {
+            if (tprev) {
+               tprev->Next = t->Next;
+            }
+            else {
+               shared->TexObjectList = t->Next;
+            }
+            break;
+         }
+         tprev = tcurr;
+         tcurr = tcurr->Next;
       }
    }
+
+   if (t->Name) {
+      /* remove from hash table */
+      HashRemove(shared->TexObjects, t->Name);
+   }
+
+   /* free texture image */
+   {
+      GLuint i;
+      for (i=0;i<MAX_TEXTURE_LEVELS;i++) {
+         if (t->Image[i]) {
+            gl_free_texture_image( t->Image[i] );
+         }
+      }
+   }
+   /* free this object */
    free( t );
 }
 
 
-/*
- * Given a texture object name, return a pointer to the texture object.
- */
-static struct gl_texture_object *
-find_texture_object( GLcontext *ctx, GLuint name )
-{
-   struct gl_texture_object *t;
-
-   assert( name>0 );
-   t = ctx->Shared->TexObjectList;
-   while (t) {
-      if (t->Name == name) {
-         return t;
-      }
-      t = t->Next;
-   }
-   return NULL;
-}
-
-
-
 
 /*
  * Examine a texture object to determine if it is complete or not.
@@ -303,12 +345,13 @@ void gl_test_texture_object_completeness( struct gl_texture_object *t )
 }
 
 
+
 /*
  * Execute glGenTextures
  */
 void gl_GenTextures( GLcontext *ctx, GLsizei n, GLuint *texName )
 {
-   GLuint i;
+   GLuint first, i;
 
    if (INSIDE_BEGIN_END(ctx)) {
       gl_error( ctx, GL_INVALID_OPERATION, "glGenTextures" );
@@ -319,8 +362,9 @@ void gl_GenTextures( GLcontext *ctx, GLsizei n, GLuint *texName )
       return;
    }
 
+   first = HashFindFreeKeyBlock(ctx->Shared->TexObjects, n);
    for (i=0;i<n;i++) {
-      texName[i] = ctx->Shared->NextFreeTextureName++;
+      texName[i] = first + i;
    }
 }
 
@@ -339,47 +383,37 @@ void gl_DeleteTextures( GLcontext *ctx, GLsizei n, const GLuint *texName)
    }
 
    for (i=0;i<n;i++) {
-      struct gl_texture_object *t, *tprev, *tcurr;
+      struct gl_texture_object *t;
       if (texName[i]>0) {
-         t = find_texture_object( ctx, texName[i] );
+         t = (struct gl_texture_object *)
+            HashLookup(ctx->Shared->TexObjects, texName[i]);
          if (t) {
             if (ctx->Texture.Current1D==t) {
                /* revert to default 1-D texture */
-               ctx->Texture.Current1D = ctx->Shared->TexObjectList;
+               ctx->Texture.Current1D = ctx->Shared->Default1D;
                t->RefCount--;
                assert( t->RefCount >= 0 );
             }
             else if (ctx->Texture.Current2D==t) {
                /* revert to default 2-D texture */
-               ctx->Texture.Current2D = ctx->Shared->TexObjectList->Next;
+               ctx->Texture.Current2D = ctx->Shared->Default2D;
                t->RefCount--;
                assert( t->RefCount >= 0 );
             }
             else if (ctx->Texture.Current3D==t) {
                /* revert to default 3-D texture */
-               ctx->Texture.Current3D = ctx->Shared->TexObjectList->Next;
+               ctx->Texture.Current3D = ctx->Shared->Default3D;
                t->RefCount--;
                assert( t->RefCount >= 0 );
             }
-            if (t->RefCount==0) {
-               /* remove texture object t from the linked list */
-               tprev = NULL;
-               tcurr = ctx->Shared->TexObjectList;
-               while (tcurr) {
-                  if (tcurr==t) {
-                     assert( tprev );
-                     tprev->Next = t->Next;
-                     gl_free_texture_object( t );
-                     break;
-                  }
-                  tprev = tcurr;
-                  tcurr = tcurr->Next;
-               }
-            }
 
             /* tell device driver to delete texture */
             if (ctx->Driver.DeleteTexture) {
-               (*ctx->Driver.DeleteTexture)( ctx, texName[i] );
+               (*ctx->Driver.DeleteTexture)( ctx, t );
+            }
+
+            if (t->RefCount==0) {
+               gl_free_texture_object(ctx->Shared, t);
             }
          }
       }
@@ -427,16 +461,13 @@ void gl_BindTexture( GLcontext *ctx, GLenum target, GLuint texName )
       /* use default n-D texture */
       switch (target) {
          case GL_TEXTURE_1D:
-            /* default 1-D texture is first in list */
-            newTexObj = ctx->Shared->TexObjectList;
+            newTexObj = ctx->Shared->Default1D;
             break;
          case GL_TEXTURE_2D:
-            /* default 2-D texture is second in list */
-            newTexObj = ctx->Shared->TexObjectList->Next;
+            newTexObj = ctx->Shared->Default2D;
             break;
          case GL_TEXTURE_3D_EXT:
-            /* default 3-D texture is third in list */
-            newTexObj = ctx->Shared->TexObjectList->Next->Next;
+            newTexObj = ctx->Shared->Default3D;
             break;
          default:
             gl_problem(ctx, "Bad target in gl_BindTexture");
@@ -444,7 +475,8 @@ void gl_BindTexture( GLcontext *ctx, GLenum target, GLuint texName )
       }
    }
    else {
-      newTexObj = find_texture_object( ctx, texName );
+      newTexObj = (struct gl_texture_object *)
+                             HashLookup(ctx->Shared->TexObjects, texName);
       if (newTexObj) {
          if (newTexObj->Dimensions!=targetDimensions) {
             /* wrong dimensionality */
@@ -454,10 +486,8 @@ void gl_BindTexture( GLcontext *ctx, GLenum target, GLuint texName )
       }
       else {
          /* create new texture object */
-         newTexObj = gl_alloc_texture_object();
-         append_texture_object( ctx->Shared->TexObjectList, newTexObj );
-         newTexObj->Name = texName;
-         newTexObj->Dimensions = targetDimensions;
+         newTexObj = gl_alloc_texture_object(ctx->Shared, texName,
+                                             targetDimensions);
       }
    }
 
@@ -497,7 +527,7 @@ void gl_BindTexture( GLcontext *ctx, GLenum target, GLuint texName )
  
    /* Pass BindTexture call to device driver */
    if (ctx->Driver.BindTexture) {
-      (*ctx->Driver.BindTexture)( ctx, target, texName );
+      (*ctx->Driver.BindTexture)( ctx, target, newTexObj );
    }
 }
 
@@ -524,7 +554,8 @@ void gl_PrioritizeTextures( GLcontext *ctx,
    for (i=0;i<n;i++) {
       struct gl_texture_object *t;
       if (texName[i]>0) {
-         t = find_texture_object( ctx, texName[i] );
+         t = (struct gl_texture_object *)
+            HashLookup(ctx->Shared->TexObjects, texName[i]);
          if (t) {
             t->Priority = CLAMP( priorities[i], 0.0F, 1.0F );
          }
@@ -559,7 +590,8 @@ GLboolean gl_AreTexturesResident( GLcontext *ctx, GLsizei n,
          gl_error( ctx, GL_INVALID_VALUE, "glAreTexturesResident(textures)" );
          return GL_FALSE;
       }
-      t = find_texture_object( ctx, texName[i] );
+      t = (struct gl_texture_object *)
+         HashLookup(ctx->Shared->TexObjects, texName[i]);
       if (t) {
          /* we consider all valid texture objects to be resident */
          residences[i] = GL_TRUE;
@@ -583,7 +615,7 @@ GLboolean gl_IsTexture( GLcontext *ctx, GLuint texture )
       gl_error( ctx, GL_INVALID_OPERATION, "glIsTextures" );
       return GL_FALSE;
    }
-   if (texture>0 && find_texture_object(ctx,texture)) {
+   if (texture>0 && HashLookup(ctx->Shared->TexObjects, texture)) {
       return GL_TRUE;
    }
    else {
