@@ -1,8 +1,8 @@
-/* $Id: texstate.c,v 1.5 1997/07/24 01:25:34 brianp Exp $ */
+/* $Id: texstate.c,v 1.8 1997/10/13 23:56:01 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,15 @@
 
 /*
  * $Log: texstate.c,v $
+ * Revision 1.8  1997/10/13 23:56:01  brianp
+ * replaced UpdateTexture() call with TexParameter()
+ *
+ * Revision 1.7  1997/09/29 23:28:14  brianp
+ * updated for new device driver texture functions
+ *
+ * Revision 1.6  1997/09/27 00:14:39  brianp
+ * added GL_EXT_paletted_texture extension
+ *
  * Revision 1.5  1997/07/24 01:25:34  brianp
  * changed precompiled header symbol from PCH to PC_HEADER
  *
@@ -245,10 +254,8 @@ void gl_TexParameterfv( GLcontext *ctx,
 
    texObj->Dirty = GL_TRUE;
 
-   /* Pass this glTexParameter*() call to the device driver. */
    if (ctx->Driver.TexParameter) {
-      GLuint ddTexObject = texObj->Name;
-      (*ctx->Driver.TexParameter)( ctx, target, ddTexObject, pname, params );
+      (*ctx->Driver.TexParameter)( ctx, target, texObj, pname, params );
    }
 }
 
@@ -257,177 +264,10 @@ void gl_TexParameterfv( GLcontext *ctx,
 void gl_GetTexLevelParameterfv( GLcontext *ctx, GLenum target, GLint level,
                                 GLenum pname, GLfloat *params )
 {
-   struct gl_texture_image *tex;
+   GLint iparam;
 
-   if (level<0 || level>=MAX_TEXTURE_LEVELS) {
-      gl_error( ctx, GL_INVALID_VALUE, "glGetTexLevelParameterfv" );
-      return;
-   }
-
-   switch (target) {
-      case GL_TEXTURE_1D:
-         tex = ctx->Texture.Current1D->Image[level];
-         switch (pname) {
-	    case GL_TEXTURE_WIDTH:
-	       *params = (GLfloat) tex->Width;
-	       break;
-	    case GL_TEXTURE_COMPONENTS:
-	       *params = (GLfloat) tex->Format;
-	       break;
-	    case GL_TEXTURE_BORDER:
-	       *params = (GLfloat) tex->Border;
-	       break;
-            case GL_TEXTURE_RED_SIZE:
-            case GL_TEXTURE_GREEN_SIZE:
-            case GL_TEXTURE_BLUE_SIZE:
-            case GL_TEXTURE_ALPHA_SIZE:
-            case GL_TEXTURE_INTENSITY_SIZE:
-            case GL_TEXTURE_LUMINANCE_SIZE:
-               *params = 8.0;  /* 8-bits */
-               break;
-	    default:
-	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameterfv(pname)" );
-	 }
-	 break;
-      case GL_TEXTURE_2D:
-         tex = ctx->Texture.Current2D->Image[level];
-	 switch (pname) {
-	    case GL_TEXTURE_WIDTH:
-	       *params = (GLfloat) tex->Width;
-	       break;
-	    case GL_TEXTURE_HEIGHT:
-	       *params = (GLfloat) tex->Height;
-	       break;
-	    case GL_TEXTURE_COMPONENTS:
-	       *params = (GLfloat) tex->Format;
-	       break;
-	    case GL_TEXTURE_BORDER:
-	       *params = (GLfloat) tex->Border;
-	       break;
-            case GL_TEXTURE_RED_SIZE:
-            case GL_TEXTURE_GREEN_SIZE:
-            case GL_TEXTURE_BLUE_SIZE:
-            case GL_TEXTURE_ALPHA_SIZE:
-            case GL_TEXTURE_INTENSITY_SIZE:
-            case GL_TEXTURE_LUMINANCE_SIZE:
-               *params = 8.0;  /* 8-bits */
-               break;
-	    default:
-	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameterfv(pname)" );
-	 }
-	 break;
-      case GL_TEXTURE_3D_EXT:
-         tex = ctx->Texture.Current3D->Image[level];
-         switch (pname) {
-            case GL_TEXTURE_WIDTH:
-               *params = (GLfloat) tex->Width;
-               break;
-            case GL_TEXTURE_HEIGHT:
-               *params = (GLfloat) tex->Height;
-               break;
-            case GL_TEXTURE_DEPTH_EXT:
-               *params = (GLfloat) tex->Depth;
-               break;
-            case GL_TEXTURE_COMPONENTS:
-               *params = (GLfloat) tex->Format;
-               break;
-            case GL_TEXTURE_BORDER:
-               *params = (GLfloat) tex->Border;
-               break;
-            default:
-               gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameterfv(pname)" );
-         }
-         break;
-      case GL_PROXY_TEXTURE_1D:
-         tex = ctx->Texture.Proxy1D->Image[level];
-         switch (pname) {
-	    case GL_TEXTURE_WIDTH:
-	       *params = (GLfloat) tex->Width;
-	       break;
-	    case GL_TEXTURE_COMPONENTS:
-	       *params = (GLfloat) tex->Format;
-	       break;
-	    case GL_TEXTURE_BORDER:
-	       *params = (GLfloat) tex->Border;
-	       break;
-            case GL_TEXTURE_RED_SIZE:
-            case GL_TEXTURE_GREEN_SIZE:
-            case GL_TEXTURE_BLUE_SIZE:
-            case GL_TEXTURE_ALPHA_SIZE:
-            case GL_TEXTURE_INTENSITY_SIZE:
-            case GL_TEXTURE_LUMINANCE_SIZE:
-               *params = 8.0;  /* 8-bits */
-               break;
-	    default:
-	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameterfv(pname)" );
-	 }
-	 break;
-      case GL_PROXY_TEXTURE_2D:
-         tex = ctx->Texture.Proxy2D->Image[level];
-	 switch (pname) {
-	    case GL_TEXTURE_WIDTH:
-	       *params = (GLfloat) tex->Width;
-	       break;
-	    case GL_TEXTURE_HEIGHT:
-	       *params = (GLfloat) tex->Height;
-	       break;
-	    case GL_TEXTURE_COMPONENTS:
-	       *params = (GLfloat) tex->Format;
-	       break;
-	    case GL_TEXTURE_BORDER:
-	       *params = (GLfloat) tex->Border;
-	       break;
-            case GL_TEXTURE_RED_SIZE:
-            case GL_TEXTURE_GREEN_SIZE:
-            case GL_TEXTURE_BLUE_SIZE:
-            case GL_TEXTURE_ALPHA_SIZE:
-            case GL_TEXTURE_INTENSITY_SIZE:
-            case GL_TEXTURE_LUMINANCE_SIZE:
-               *params = 8.0;  /* 8-bits */
-               break;
-	    default:
-	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameterfv(pname)" );
-	 }
-	 break;
-      case GL_PROXY_TEXTURE_3D_EXT:
-         tex = ctx->Texture.Proxy3D->Image[level];
-	 switch (pname) {
-	    case GL_TEXTURE_WIDTH:
-	       *params = (GLfloat) tex->Width;
-	       break;
-	    case GL_TEXTURE_HEIGHT:
-	       *params = (GLfloat) tex->Height;
-	       break;
-	    case GL_TEXTURE_DEPTH_EXT:
-	       *params = (GLfloat) tex->Depth;
-	       break;
-	    case GL_TEXTURE_COMPONENTS:
-	       *params = (GLfloat) tex->Format;
-	       break;
-	    case GL_TEXTURE_BORDER:
-	       *params = (GLfloat) tex->Border;
-	       break;
-            case GL_TEXTURE_RED_SIZE:
-            case GL_TEXTURE_GREEN_SIZE:
-            case GL_TEXTURE_BLUE_SIZE:
-            case GL_TEXTURE_ALPHA_SIZE:
-            case GL_TEXTURE_INTENSITY_SIZE:
-            case GL_TEXTURE_LUMINANCE_SIZE:
-               *params = 8.0;  /* 8-bits */
-               break;
-	    default:
-	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameterfv(pname)" );
-	 }
-	 break;
-     default:
-	 gl_error( ctx, GL_INVALID_ENUM, "glGetTexLevelParameterfv(target)" );
-   }	 
+   gl_GetTexLevelParameteriv( ctx, target, level, pname, &iparam );
+   *params = (GLfloat) iparam;
 }
 
 
@@ -438,7 +278,7 @@ void gl_GetTexLevelParameteriv( GLcontext *ctx, GLenum target, GLint level,
    struct gl_texture_image *tex;
 
    if (level<0 || level>=MAX_TEXTURE_LEVELS) {
-      gl_error( ctx, GL_INVALID_VALUE, "glGetTexLevelParameteriv" );
+      gl_error( ctx, GL_INVALID_VALUE, "glGetTexLevelParameter[if]v" );
       return;
    }
 
@@ -463,9 +303,12 @@ void gl_GetTexLevelParameteriv( GLcontext *ctx, GLenum target, GLint level,
             case GL_TEXTURE_LUMINANCE_SIZE:
                *params = 8;  /* 8-bits */
                break;
+            case GL_TEXTURE_INDEX_SIZE_EXT:
+               *params = 8;
+               break;
 	    default:
 	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameteriv(pname)" );
+                         "glGetTexLevelParameter[if]v(pname)" );
 	 }
 	 break;
       case GL_TEXTURE_2D:
@@ -491,9 +334,12 @@ void gl_GetTexLevelParameteriv( GLcontext *ctx, GLenum target, GLint level,
             case GL_TEXTURE_LUMINANCE_SIZE:
                *params = 8;  /* 8-bits */
                break;
+            case GL_TEXTURE_INDEX_SIZE_EXT:
+               *params = 8;
+               break;
 	    default:
 	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameteriv(pname)" );
+                         "glGetTexLevelParameter[if]v(pname)" );
 	 }
 	 break;
       case GL_TEXTURE_3D_EXT:
@@ -514,9 +360,12 @@ void gl_GetTexLevelParameteriv( GLcontext *ctx, GLenum target, GLint level,
             case GL_TEXTURE_BORDER:
                *params = tex->Border;
                break;
+            case GL_TEXTURE_INDEX_SIZE_EXT:
+               *params = 8;
+               break;
             default:
                gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameteriv(pname)" );
+                         "glGetTexLevelParameter[if]v(pname)" );
          }
          break;
       case GL_PROXY_TEXTURE_1D:
@@ -539,9 +388,12 @@ void gl_GetTexLevelParameteriv( GLcontext *ctx, GLenum target, GLint level,
             case GL_TEXTURE_LUMINANCE_SIZE:
                *params = 8;  /* 8-bits */
                break;
+            case GL_TEXTURE_INDEX_SIZE_EXT:
+               *params = 8;
+               break;
 	    default:
 	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameteriv(pname)" );
+                         "glGetTexLevelParameter[if]v(pname)" );
 	 }
 	 break;
       case GL_PROXY_TEXTURE_2D:
@@ -567,9 +419,12 @@ void gl_GetTexLevelParameteriv( GLcontext *ctx, GLenum target, GLint level,
             case GL_TEXTURE_LUMINANCE_SIZE:
                *params = 8;  /* 8-bits */
                break;
+            case GL_TEXTURE_INDEX_SIZE_EXT:
+               *params = 8;
+               break;
 	    default:
 	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameteriv(pname)" );
+                         "glGetTexLevelParameter[if]v(pname)" );
 	 }
 	 break;
       case GL_PROXY_TEXTURE_3D_EXT:
@@ -598,13 +453,16 @@ void gl_GetTexLevelParameteriv( GLcontext *ctx, GLenum target, GLint level,
             case GL_TEXTURE_LUMINANCE_SIZE:
                *params = 8;  /* 8-bits */
                break;
+            case GL_TEXTURE_INDEX_SIZE_EXT:
+               *params = 8;
+               break;
 	    default:
 	       gl_error( ctx, GL_INVALID_ENUM,
-                         "glGetTexLevelParameteriv(pname)" );
+                         "glGetTexLevelParameter[if]v(pname)" );
 	 }
 	 break;
      default:
-	 gl_error( ctx, GL_INVALID_ENUM, "glGetTexLevelParameteriv(target)" );
+	 gl_error(ctx, GL_INVALID_ENUM, "glGetTexLevelParameter[if]v(target)");
    }	 
 }
 
