@@ -1,8 +1,8 @@
-/* $Id: types.h,v 1.38 1997/08/13 01:30:55 brianp Exp $ */
+/* $Id: types.h,v 1.46 1997/10/29 01:33:29 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,30 @@
 
 /*
  * $Log: types.h,v $
+ * Revision 1.46  1997/10/29 01:33:29  brianp
+ * added DriverMgrCtx field to gl_context struct
+ *
+ * Revision 1.45  1997/10/29 01:29:09  brianp
+ * added GL_EXT_point_parameters extension from Daniel Barrero
+ *
+ * Revision 1.44  1997/10/16 01:59:08  brianp
+ * added GL_EXT_shared_texture_palette extension
+ *
+ * Revision 1.43  1997/10/16 01:13:03  brianp
+ * removed teximage Dirty, DirtyPalette flags, mondello code
+ *
+ * Revision 1.42  1997/09/29 23:27:08  brianp
+ * added Dirty and DriverData fields for texturing
+ *
+ * Revision 1.41  1997/09/27 00:13:44  brianp
+ * added GL_EXT_paletted_texture extension
+ *
+ * Revision 1.40  1997/09/23 00:57:38  brianp
+ * now using hash table for texture objects
+ *
+ * Revision 1.39  1997/09/22 02:33:58  brianp
+ * display lists now implemented with hash table
+ *
  * Revision 1.38  1997/08/13 01:30:55  brianp
  * LightTwoSide is now a GLboolean
  *
@@ -150,6 +174,8 @@
 #include "config.h"
 
 
+struct HashTable;
+
 
 /*
  * Accumulation buffer data type:
@@ -239,12 +265,12 @@ struct gl_image {
         GLenum Format;		/* GL_COLOR_INDEX, GL_RED, GL_RGB, etc */
 	GLenum Type;		/* GL_UNSIGNED_BYTE or GL_FLOAT or GL_BITMAP */
 	GLvoid *Data;
-	GLboolean Interleaved;	/* If TRUE and Format==GL_RGB, GL_RGBA or
-				 * GL_LUMINANCE_ALPHA then each row is
-				 * stored as RRR..RGGG..GBBB.B instead of
-				 * RGBRGBRGBRGB..RGB.  This is only used
-				 * for glDrawPixels.
-				 */
+	GLboolean Interleaved;  /* If TRUE and Format==GL_RGB, GL_RGBA or
+                                 * GL_LUMINANCE_ALPHA then each row is
+                                 * stored as RRR..RGGG..GBBB.B instead of
+                                 * RGBRGBRGBRGB..RGB.  This is only used
+                                 * for glDrawPixels.
+                                 */
 	GLint RefCount;
 };
 
@@ -253,8 +279,10 @@ struct gl_image {
 /* Texture image record */
 struct gl_texture_image {
 	GLenum Format;		/* GL_ALPHA, GL_LUMINANCE, GL_LUMINANCE_ALPHA,
-				 * GL_INTENSITY, GL_RGB, or GL_RGBA
+				 * GL_INTENSITY, GL_RGB, GL_RGBA, or
+				 * GL_COLOR_INDEX
 				 */
+	GLenum IntFormat;	/* Internal format as given by the user */
 	GLuint Border;		/* 0 or 1 */
 	GLuint Width;		/* = 2^WidthLog2 + 2*Border */
 	GLuint Height;		/* = 2^HeightLog2 + 2*Border */
@@ -267,6 +295,9 @@ struct gl_texture_image {
 	GLuint DepthLog2;	/* = log2(Depth2) */
 	GLuint MaxLog2;		/* = MAX(WidthLog2, HeightLog2) */
 	GLubyte *Data;		/* Image data as unsigned bytes */
+
+	/* For device driver: */
+	void *DriverData;	/* Arbitrary device driver data */
 };
 
 
@@ -306,6 +337,8 @@ struct gl_api_table {
 			GLboolean, GLboolean, GLboolean, GLboolean );
    void (*ColorMaterial)( GLcontext *, GLenum, GLenum );
    void (*ColorPointer)( GLcontext *, GLint, GLenum, GLsizei, const GLvoid * );
+   void (*ColorTable)( GLcontext *, GLenum, GLenum, struct gl_image * );
+   void (*ColorSubTable)( GLcontext *, GLenum, GLsizei, struct gl_image * );
    void (*CopyPixels)( GLcontext *, GLint, GLint, GLsizei, GLsizei, GLenum );
    void (*CopyTexImage1D)( GLcontext *, GLenum, GLint, GLenum,
                            GLint, GLint, GLsizei, GLint );
@@ -356,6 +389,8 @@ struct gl_api_table {
    void (*GenTextures)( GLcontext *, GLsizei, GLuint * );
    void (*GetBooleanv)( GLcontext *, GLenum, GLboolean * );
    void (*GetClipPlane)( GLcontext *, GLenum, GLdouble * );
+   void (*GetColorTable)( GLcontext *, GLenum, GLenum, GLenum, GLvoid *);
+   void (*GetColorTableParameteriv)( GLcontext *, GLenum, GLenum, GLint *);
    void (*GetDoublev)( GLcontext *, GLenum, GLdouble * );
    GLenum (*GetError)( GLcontext * );
    void (*GetFloatv)( GLcontext *, GLenum, GLfloat * );
@@ -431,6 +466,7 @@ struct gl_api_table {
    void (*PixelStorei)( GLcontext *, GLenum, GLint );
    void (*PixelTransferf)( GLcontext *, GLenum, GLfloat );
    void (*PixelZoom)( GLcontext *, GLfloat, GLfloat );
+   void (*PointParameterfvEXT)( GLcontext *, GLenum, const GLfloat * );
    void (*PointSize)( GLcontext *, GLfloat );
    void (*PolygonMode)( GLcontext *, GLenum, GLenum );
    void (*PolygonOffset)( GLcontext *, GLfloat, GLfloat );
@@ -830,6 +866,11 @@ struct gl_pixel_attrib {
 struct gl_point_attrib {
 	GLboolean SmoothFlag;	/* True if GL_POINT_SMOOTH is enabled */
 	GLfloat Size;		/* Point size */
+	GLfloat Params[3];	/* Point Parameters EXT distance atenuation 
+				   factors by default = {1,0,0} */ 
+	GLfloat MinSize;	/* Default 0.0 always >=0 */
+	GLfloat MaxSize;	/* Default MAX_POINT_SIZE */
+	GLfloat Threshold;	/* Default 1.0 */ 
 };
 
 
@@ -908,6 +949,12 @@ struct gl_texture_attrib {
 	struct gl_texture_object *Proxy2D;
 	struct gl_texture_object *Proxy3D;
 #endif
+	/* GL_EXT_shared_texture_palette */
+	GLboolean SharedPalette;
+	GLubyte Palette[MAX_TEXTURE_PALETTE_SIZE*4];
+	GLuint PaletteSize;
+	GLenum PaletteIntFormat;
+	GLenum PaletteFormat;
 };
 
 
@@ -1088,27 +1135,35 @@ struct gl_texture_object {
 	GLenum MagFilter;		/* magnification filter */
 	GLfloat MinMagThresh;		/* min/mag threshold */
 	struct gl_texture_image *Image[MAX_TEXTURE_LEVELS];
-	GLboolean Dirty;		/* New values? */
+
+	/* GL_EXT_paletted_texture */
+	GLubyte Palette[MAX_TEXTURE_PALETTE_SIZE*4];
+	GLuint PaletteSize;
+	GLenum PaletteIntFormat;
+	GLenum PaletteFormat;
+
+	/* For device driver: */
+	GLboolean Dirty;	/* Set when any texobj state changes */
+	void *DriverData;	/* Arbitrary device driver data */
+
 	GLboolean Complete;		/* Complete set of images? */
 	TextureSampleFunc SampleFunc;
 	struct gl_texture_object *Next;	/* Next in linked list */
 };
 
 
-/*
- * Display list building block:
- */
-union node;
-
 
 /*
  * State which can be shared by multiple contexts:
  */
 struct gl_shared_state {
-	GLint RefCount;				/* Reference count */
-	union node *List[MAX_DISPLAYLISTS];	/* Display lists */
-	struct gl_texture_object *TexObjectList;/* Texture objects */
-	GLuint NextFreeTextureName;		/* Used by glGenTextures */
+   GLint RefCount;			/* Reference count */
+   struct HashTable *DisplayList;	/* Display lists hash table */
+   struct HashTable *TexObjects;	/* Texture objects hash table */
+   struct gl_texture_object *TexObjectList;/* Linked list of texture objects */
+   struct gl_texture_object *Default1D;	/* Default texture objects */
+   struct gl_texture_object *Default2D;
+   struct gl_texture_object *Default3D;
 };
 
 
@@ -1186,7 +1241,7 @@ struct gl_frame_buffer {
 
 /*
  * Bitmasks to indicate what auxillary information must be interpolated
- * when clipping (CC.ClipMask).
+ * when clipping (ClipMask).
  */
 #define CLIP_FCOLOR_BIT		0x01
 #define CLIP_BCOLOR_BIT		0x02
@@ -1197,7 +1252,7 @@ struct gl_frame_buffer {
 
 
 /*
- * Bitmasks to indicate which rasterization options are enabled (CC.RasterMask)
+ * Bitmasks to indicate which rasterization options are enabled (RasterMask)
  */
 #define ALPHATEST_BIT		0x001	/* Alpha-test pixels */
 #define BLEND_BIT		0x002	/* Blend pixels */
@@ -1214,7 +1269,7 @@ struct gl_frame_buffer {
 
 
 /*
- * Bits to indicate what state has to be updated (CC.NewState)
+ * Bits to indicate what state has to be updated (NewState)
  */
 #define NEW_LIGHTING	0x1
 #define NEW_RASTER_OPS	0x2
@@ -1255,8 +1310,8 @@ struct gl_context {
 	/* Driver function pointer table */
 	struct dd_function_table Driver;
 
-	/* Opaque handle for the device driver context */
-	void *DriverCtx;
+	void *DriverCtx;	/* Points to device driver context/state */
+	void *DriverMgrCtx;	/* Points to device driver manager (optional)*/
 
 	/* Modelview matrix and stack */
 	GLboolean NewModelViewMatrix;
@@ -1325,6 +1380,8 @@ struct gl_context {
 
 	GLenum ErrorValue;		/* Last error code */
 
+	GLboolean DirectContext;	/* Important for real GLX */
+
 	/* Miscellaneous */
         GLuint NewState;        /* bitwise OR of NEW_* flags */
 	GLenum RenderMode;	/* either GL_RENDER, GL_SELECT, GL_FEEDBACK */
@@ -1345,9 +1402,6 @@ struct gl_context {
 	/* Current Primitive functions */
         points_func PointsFunc;
         line_func LineFunc;
-#ifdef MONDELLO
-        line_func LineFunc2;
-#endif
         triangle_func TriangleFunc;
         quad_func QuadFunc;
         rect_func RectFunc;
