@@ -1,8 +1,8 @@
-/* $Id: varray.c,v 1.14 1997/06/20 02:50:19 brianp Exp $ */
+/* $Id: varray.c,v 1.16 1997/08/14 01:15:47 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.3
+ * Version:  2.4
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,12 @@
 
 /*
  * $Log: varray.c,v $
+ * Revision 1.16  1997/08/14 01:15:47  brianp
+ * gl_ArrayElement()'s assignment of current normal was wrong
+ *
+ * Revision 1.15  1997/07/24 01:25:54  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
  * Revision 1.14  1997/06/20 02:50:19  brianp
  * replaced Current.IntColor with Current.ByteColor
  *
@@ -79,7 +85,7 @@
  */
 
 
-#ifdef PCH
+#ifdef PC_HEADER
 #include "all.h"
 #else
 #include <stdlib.h>
@@ -349,8 +355,8 @@ void gl_ArrayElement( GLcontext *ctx, GLint i )
    }
    else {
       VB->Normal[count][0] = ctx->Current.Normal[0];
-      VB->Normal[count][1] = ctx->Current.Normal[0];
-      VB->Normal[count][2] = ctx->Current.Normal[0];
+      VB->Normal[count][1] = ctx->Current.Normal[1];
+      VB->Normal[count][2] = ctx->Current.Normal[2];
    } 
 
    /* TODO: directly set VB->Fcolor instead of calling a glColor command */
