@@ -1,8 +1,8 @@
-/* $Id: vbrender.c,v 1.15 1997/09/18 01:32:47 brianp Exp $ */
+/* $Id: vbrender.c,v 1.16 1997/11/20 00:00:47 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,9 @@
 
 /*
  * $Log: vbrender.c,v $
+ * Revision 1.16  1997/11/20 00:00:47  brianp
+ * only call Driver.RasterSetup() once in render_clipped_polygon()
+ *
  * Revision 1.15  1997/09/18 01:32:47  brianp
  * fixed divide by zero problem for "weird" projection matrices
  *
@@ -480,12 +483,9 @@ static void render_clipped_polygon( GLcontext *ctx, GLuint n, GLuint vlist[] )
             win[j][0] = win[j][1] = win[j][2] = 0.0F;
          }
       }
-      if (ctx->Driver.RasterSetup) {
-         /* Device driver rasterization setup */
-         for (i=0;i<n;i++) {
-            GLuint j = vlist[i];
-            (*ctx->Driver.RasterSetup)( ctx, j, j+1 );
-         }
+      if (ctx->Driver.RasterSetup && (VB->Free > VB_MAX)) {
+         /* Device driver raster setup for newly introduced vertices */
+         (*ctx->Driver.RasterSetup)(ctx, VB_MAX, VB->Free);
       }
    }
 
