@@ -1,8 +1,8 @@
-/* $Id: vbxform.c,v 1.15 1997/09/18 01:32:47 brianp Exp $ */
+/* $Id: vbxform.c,v 1.20 1997/11/20 00:09:38 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,21 @@
 
 /*
  * $Log: vbxform.c,v $
+ * Revision 1.20  1997/11/20 00:09:38  brianp
+ * transform_points4() wasn't calling asm routines
+ *
+ * Revision 1.19  1997/10/30 06:00:06  brianp
+ * added Intel X86 assembly optimzations (Josh Vanderhoof)
+ *
+ * Revision 1.18  1997/10/15 00:36:36  brianp
+ * renamed the FAST/REGULAR_MATH macros
+ *
+ * Revision 1.17  1997/10/04 00:30:52  brianp
+ * vertices specified with glVertex4 weren't always projected correctly
+ *
+ * Revision 1.16  1997/09/29 22:24:22  brianp
+ * added REGULAR/FAST_MATH macros
+ *
  * Revision 1.15  1997/09/18 01:32:47  brianp
  * fixed divide by zero problem for "weird" projection matrices
  *
@@ -91,6 +106,7 @@
 #include "light.h"
 #include "macros.h"
 #include "matrix.h"
+#include "mmath.h"
 #include "shade.h"
 #include "texture.h"
 #include "types.h"
@@ -206,6 +222,8 @@ static void transform_points2( GLcontext *ctx, GLuint n,
 static void transform_points3( GLcontext *ctx, GLuint n,
                                /*const*/ GLfloat vObj[][4], GLfloat vEye[][4] )
 {
+#ifndef USE_ASM
+   START_FAST_MATH;
    switch (ctx->ModelViewMatrixType) {
       case MATRIX_GENERAL:
          {
@@ -283,8 +301,32 @@ static void transform_points3( GLcontext *ctx, GLuint n,
       default:
          /* should never get here */
          gl_problem( NULL, "invalid matrix type in transform_points3()" );
+   }
+   END_FAST_MATH;
+#else
+   switch (ctx->ModelViewMatrixType) {
+      case MATRIX_GENERAL:
+         asm_transform_points3_general( n, vEye, ctx->ModelViewMatrix, vObj );
+         break;
+      case MATRIX_IDENTITY:
+         asm_transform_points3_identity( n, vEye, vObj );
+         break;
+      case MATRIX_2D:
+         asm_transform_points3_2d( n, vEye, ctx->ModelViewMatrix, vObj );
+         break;
+      case MATRIX_2D_NO_ROT:
+         asm_transform_points3_2d_no_rot( n, vEye, ctx->ModelViewMatrix,
+                                          vObj );
+         break;
+      case MATRIX_3D:
+         asm_transform_points3_3d( n, vEye, ctx->ModelViewMatrix, vObj );
+         break;
+      default:
+         /* should never get here */
+         gl_problem( NULL, "invalid matrix type in transform_points3()" );
          return;
    }
+#endif
 }
 
 
@@ -300,6 +342,8 @@ static void transform_points3( GLcontext *ctx, GLuint n,
 static void transform_points4( GLcontext *ctx, GLuint n,
                                /*const*/ GLfloat vObj[][4], GLfloat vEye[][4] )
 {
+#ifndef USE_ASM
+   START_FAST_MATH;
    switch (ctx->ModelViewMatrixType) {
       case MATRIX_GENERAL:
          {
@@ -381,8 +425,32 @@ static void transform_points4( GLcontext *ctx, GLuint n,
       default:
          /* should never get here */
          gl_problem( NULL, "invalid matrix type in transform_points4()" );
+   }
+   END_FAST_MATH;
+#else
+   switch (ctx->ModelViewMatrixType) {
+      case MATRIX_GENERAL:
+         asm_transform_points4_general( n, vEye, ctx->ModelViewMatrix, vObj );
+         break;
+      case MATRIX_IDENTITY:
+         asm_transform_points4_identity( n, vEye, vObj );
+         break;
+      case MATRIX_2D:
+         asm_transform_points4_2d( n, vEye, ctx->ModelViewMatrix, vObj );
+         break;
+      case MATRIX_2D_NO_ROT:
+         asm_transform_points4_2d_no_rot( n, vEye, ctx->ModelViewMatrix,
+                                          vObj );
+         break;
+      case MATRIX_3D:
+         asm_transform_points4_3d( n, vEye, ctx->ModelViewMatrix, vObj );
+         break;
+      default:
+         /* should never get here */
+         gl_problem( NULL, "invalid matrix type in transform_points4()" );
          return;
    }
+#endif
 }
 
 
@@ -395,6 +463,8 @@ static void transform_points4( GLcontext *ctx, GLuint n,
  */
 static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
 {
+#ifndef USE_ASM
+   START_FAST_MATH;
    switch (ctx->TextureMatrixType) {
       case MATRIX_GENERAL:
          {
@@ -450,8 +520,28 @@ static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
       default:
          /* should never get here */
          gl_problem( NULL, "invalid matrix type in transform_texcoords()" );
+   }
+   END_FAST_MATH;
+#else
+   switch (ctx->TextureMatrixType) {
+      case MATRIX_GENERAL:
+         asm_transform_points4_general( n, t, ctx->TextureMatrix, t );
+         break;
+      case MATRIX_IDENTITY:
+         /* Do nothing */
+         break;
+      case MATRIX_2D:
+         asm_transform_points4_2d( n, t, ctx->TextureMatrix, t );
+         break;
+      case MATRIX_3D:
+         asm_transform_points4_3d( n, t, ctx->TextureMatrix, t );
+         break;
+      default:
+         /* should never get here */
+         gl_problem( NULL, "invalid matrix type in transform_texcoords()" );
          return;
    }
+#endif
 }
 
 
@@ -472,9 +562,11 @@ static void project_and_cliptest( GLcontext *ctx,
                                   GLubyte *orMask, GLubyte *andMask )
 
 {
+#ifndef USE_ASM
    GLubyte tmpOrMask = *orMask;
    GLubyte tmpAndMask = *andMask;
 
+   START_FAST_MATH;
    switch (ctx->ProjectionMatrixType) {
       case MATRIX_GENERAL:
          {
@@ -600,11 +692,34 @@ static void project_and_cliptest( GLcontext *ctx,
       default:
          /* should never get here */
          gl_problem( NULL, "invalid matrix type in project_and_cliptest()" );
-         return;
    }
 
    *orMask = tmpOrMask;
    *andMask = tmpAndMask;
+   END_FAST_MATH;
+#else
+   switch (ctx->ProjectionMatrixType) {
+      case MATRIX_GENERAL:
+         asm_project_and_cliptest_general( n, vClip, ctx->ProjectionMatrix, vEye, 
+                                           clipMask, orMask, andMask );
+         break;
+      case MATRIX_IDENTITY:
+         asm_project_and_cliptest_identity( n, vClip, vEye, clipMask, orMask, andMask );
+         break;
+      case MATRIX_ORTHO:
+         asm_project_and_cliptest_ortho( n, vClip, ctx->ProjectionMatrix, vEye, 
+                                         clipMask, orMask, andMask );
+         break;
+      case MATRIX_PERSPECTIVE:
+         asm_project_and_cliptest_perspective( n, vClip, ctx->ProjectionMatrix,
+                                               vEye, clipMask, orMask, andMask );
+         break;
+      default:
+         /* should never get here */
+         gl_problem( NULL, "invalid matrix type in project_and_cliptest()" );
+         return;
+   }
+#endif
 }
 
 
@@ -628,6 +743,8 @@ static GLuint userclip_vertices( GLcontext *ctx, GLuint n,
 
    ASSERT(ctx->Transform.AnyClip);
 
+   START_FAST_MATH;
+
    for (p=0;p<MAX_CLIP_PLANES;p++) {
       if (ctx->Transform.ClipEnabled[p]) {
          GLfloat a = ctx->Transform.ClipEquation[p][0];
@@ -655,6 +772,8 @@ static GLuint userclip_vertices( GLcontext *ctx, GLuint n,
       }
    }
 
+   END_FAST_MATH;
+
    return anyClipped ? CLIP_SOME : CLIP_NONE;
 }
 
@@ -679,9 +798,13 @@ static void viewport_map_vertices( GLcontext *ctx,
    GLfloat ty = ctx->Viewport.Ty;
    GLfloat sz = ctx->Viewport.Sz;
    GLfloat tz = ctx->Viewport.Tz;
+
+   START_FAST_MATH;
+
    if ((ctx->ProjectionMatrixType==MATRIX_ORTHO || 
         ctx->ProjectionMatrixType==MATRIX_IDENTITY)
-       && ctx->ModelViewMatrixType!=MATRIX_GENERAL) {
+       && ctx->ModelViewMatrixType!=MATRIX_GENERAL
+       && (ctx->VB->VertexSizeMask & VERTEX4_BIT)==0) {
       /* don't need to divide by W */
       if (clipMask) {
          /* one or more vertices are clipped */
@@ -745,6 +868,8 @@ static void viewport_map_vertices( GLcontext *ctx,
          }
       }
    }
+
+   END_FAST_MATH;
 }
 
 
