@@ -1,8 +1,8 @@
-/* $Id: xmesa1.c,v 1.37 1997/08/26 02:08:42 brianp Exp $ */
+/* $Id: xmesa1.c,v 1.39 1997/11/07 03:48:03 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,12 @@
 
 /*
  * $Log: xmesa1.c,v $
+ * Revision 1.39  1997/11/07 03:48:03  brianp
+ * added basic stencil buffer and accumulation buffer support for 3Dfx/Glide
+ *
+ * Revision 1.38  1997/11/02 20:21:07  brianp
+ * if a window doesn't have a colormap, allocate one (xlockmore)
+ *
  * Revision 1.37  1997/08/26 02:08:42  brianp
  * replaced MESA_FX_WINDOW/FULLSCREEN env vars with MESA_GLX_FX
  *
@@ -1508,18 +1514,27 @@ XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
             FXMESA_DEPTH_SIZE, 1,
             FXMESA_NONE
          };
-
          c->FXctx = fxMesaCreateBestContext(0, FX_WIDTH, FX_HEIGHT, attribs);
          c->FXhack = c->FXctx ? GL_TRUE : GL_FALSE;
       }
       else if (tolower(fxEnvVar[0])=='f') {
          /* Full-screen rendering */
-         int attribs[] = {
-            FXMESA_DOUBLEBUFFER,
-            FXMESA_DEPTH_SIZE, 1,
-            FXMESA_NONE
-         };
-
+         int attribs[100];
+         int numAttribs = 0;
+         attribs[numAttribs++] = FXMESA_DEPTH_SIZE;
+         attribs[numAttribs++] = 1;
+         if (v->gl_visual->DBflag) {
+            attribs[numAttribs++] = FXMESA_DOUBLEBUFFER;
+         }
+         if (v->gl_visual->AccumBits > 0) {
+            attribs[numAttribs++] = FXMESA_ACCUM_SIZE;
+            attribs[numAttribs++] = v->gl_visual->AccumBits;
+         }
+         if (v->gl_visual->StencilBits > 0) {
+            attribs[numAttribs++] = FXMESA_STENCIL_SIZE;
+            attribs[numAttribs++] = v->gl_visual->StencilBits;
+         }
+         attribs[numAttribs++] = FXMESA_NONE;
          c->FXctx = fxMesaCreateBestContext(0, FX_WIDTH, FX_HEIGHT, attribs);
          c->FXuse = c->FXctx ? GL_TRUE : GL_FALSE;
          c->FXhack = GL_FALSE;
@@ -1574,7 +1589,17 @@ XMesaBuffer XMesaCreateWindowBuffer( XMesaVisual v, Window w )
    b->xm_visual = v;
    b->pixmap_flag = GL_FALSE;
    b->display = v->display;
-   b->cmap = attr.colormap;
+   if (attr.colormap) {
+      b->cmap = attr.colormap;
+   }
+   else {
+      if (getenv("MESA_DEBUG")) {
+         fprintf(stderr, "Window %d has no colormap!\n", w);
+      }
+      /* this is weird, a window w/out a colormap!? */
+      /* OK, let's just allocate a new one and hope for the best */
+      b->cmap = XCreateColormap(v->display, w, attr.visual, AllocNone);
+   }
 
    /* determine back buffer implementation */
    if (v->gl_visual->DBflag) {
@@ -1596,7 +1621,7 @@ XMesaBuffer XMesaCreateWindowBuffer( XMesaVisual v, Window w )
    }
 
    if (!initialize_visual_and_buffer( v, b, v->gl_visual->RGBAflag,
-                                      w, attr.colormap )) {
+                                      w, b->cmap )) {
       gl_destroy_framebuffer( b->gl_buffer );
       free_xmesa_buffer(b);
       return NULL;
